# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pytz
import sqlalchemy
from sqlalchemy import and_
from sqlalchemy import or_

from oslo_db.sqlalchemy import utils as sqlalchemyutils
from oslo_log import log
from oslo_utils import timeutils

from vitrage.common.constants import EdgeLabel as ELabel
from vitrage.common.constants import HistoryProps as HProps
from vitrage.common.exception import VitrageInputError
from vitrage.entity_graph.mappings.operational_alarm_severity import \
    OperationalAlarmSeverity as OSeverity
from vitrage.storage import db_time
from vitrage.storage.sqlalchemy import models

LOG = log.getLogger(__name__)


LIMIT = 10000
ASC = 'asc'
DESC = 'desc'


class HistoryFacadeConnection(object):
    def __init__(self, engine_facade, alarms, edges, changes):
        self._engine_facade = engine_facade
        self._alarms = alarms
        self._edges = edges
        self._changes = changes

    def disable_alarms_in_history(self):
        end_time = db_time()
        active_alarms = self.get_alarms(limit=0)
        changes_to_add = [alarm.vitrage_id for alarm in active_alarms]
        self._alarms.end_all_alarms(end_time)
        self._edges.end_all_edges(end_time)
        self._changes.add_end_changes(changes_to_add, end_time)

    @staticmethod
    def add_utc_timezone(time):
        time = pytz.utc.localize(time)
        return time

    def count_active_alarms(self, project_id=None, is_admin_project=False):

        session = self._engine_facade.get_session()
        query = session.query(models.Alarm)
        query = query.filter(models.Alarm.end_timestamp > db_time())
        query = self._add_project_filtering_to_query(
            query, project_id, is_admin_project)

        query_severe = query.filter(
            models.Alarm.vitrage_operational_severity == OSeverity.SEVERE)
        query_critical = query.filter(
            models.Alarm.vitrage_operational_severity == OSeverity.CRITICAL)
        query_warning = query.filter(
            models.Alarm.vitrage_operational_severity == OSeverity.WARNING)
        query_ok = query.filter(
            models.Alarm.vitrage_operational_severity == OSeverity.OK)
        query_na = query.filter(
            models.Alarm.vitrage_operational_severity == OSeverity.NA)

        counts = {OSeverity.SEVERE: query_severe.count(),
                  OSeverity.CRITICAL: query_critical.count(),
                  OSeverity.WARNING: query_warning.count(),
                  OSeverity.OK: query_ok.count(),
                  OSeverity.NA: query_na.count()}

        return counts

    def get_alarms(self,
                   start=None,
                   end=None,
                   limit=LIMIT,
                   sort_by=(HProps.START_TIMESTAMP, HProps.VITRAGE_ID),
                   sort_dirs=(ASC, ASC),
                   filter_by=None,
                   filter_vals=None,
                   next_page=True,
                   marker=None,
                   only_active_alarms=False,
                   project_id=None,
                   is_admin_project=False):
        """Return alarms that match all filters sorted by the given keys.

        Deleted alarms will be returned when only_active_alarms=False.

        filtering and sorting are possible on each row of alarms table
        (pay attantion: it is not recommended to filter by start_timestamp
        and end_timestamp when start or end arguments are passed):
        vitrage_id,
        start_timestamp,
        end_timestamp,
        name,
        vitrage_type,
        vitrage_aggregated_severity,
        project_id,
        vitrage_resource_type,
        vitrage_resource_id,
        vitrage_resource_project_id,
        payload

        Time Frame:
        start and end arguments gives the time frame for required alarms.
        Required format is the format that can be parsed by timeutils library.
        If both arguments are given, returned alarms are the alarms that
        where active sometime during given time frame
        (including active and inactive alarms):

        1.  start_ts------------end_ts
        2.                   start_ts------------end_ts
        3.                                  start_ts------------end_ts
        4.      start_ts---------------------------------------end_ts
                        start                            end
                          |_______________________________|

        If only start is given, all alarms that started after this time
        will be returned (including active and inactive alarms):
        1.                   start_ts------------end_ts
        2.                                  start_ts------
                        start                            now
                          |_______________________________|

        note1: end argument can't be used without start argument
        note2: time frame can't be used with flag only_active_alarms=True

        Filtering:
        filter_by represents parameters to filter on,
        and filter_vals contains the values to filter on in corresponding
        order to the order of parameters in filter_by.
        The filtering is according to SQL 'like' statement.
        It's possible to filter on each row of alarms table
        The filtering is also possible on list of values.

        examples:
        1. In the following example:
            |   filter_by = ['vitrage_type', 'vitrage_resource_type']
            |   filter_vals = ['zabbix', 'nova']
            which will be evaluated to:
                Alarm.vitrage_type like '%zabbix%'
                and Alarm.vitrage_resource_type like '%nova%'
            Tthe filtering will be done so the query returns all the alarms
            in the DB with vitrage type containing the string 'zabbix'
            and vitrage resource type containing the string 'nova'

        2. Following example is filtering list of values for one same property:
            |   filter_by = ['vitrage_type', 'vitrage_id']
            |   filter_vals = ['zabbix', ['123', '456', '789']]
            It will be evaluated to:
                Alarm.vitrage_type like '%zabbix%'
                and Alarm.vitrage_resource_type like '%123%'
                    or like '%456%'
                    or like '%789%'
            Tthe filtering will be done so the query returns all the alarms
            in the DB with vitrage type containing the string 'zabbix'
            and with one of vitrage_ids that are in the list in filter_vals[1]


        :param start: start of time frame
        :param end: end of time frame
        :param limit: maximum number of items to return,
        if limit=0 the method will return all matched items in alarms table,
        if limit is bigger then default parameter LIMIT, the number of items
        that will be returned will be defined by the default parameter LIMIT
        :param sort_by: array of attributes by which results should be sorted
        :param sort_dirs: per-column array of sort_dirs,
        corresponding to sort_keys ('asc' or 'desc').
        :param filter_by: array of attributes by which results will be filtered
        :param filter_vals: per-column array of filter values
        corresponding to filter_by
        :param next_page: if True will return next page when marker is given,
         if False will return previous page when marker is given,
         otherwise, returns first page if no marker was given.
        :param marker: if None returns first page, else if vitrage_id is given
        and next_page is True, return next #limit results after marker,
        else, if next page is False,return #limit results before marker.
        :param only_active_alarms: if True, returns only active alarms,
        if False return active and non-active alarms.
        :param project_id: if None there is no filtering by project_id
        (equals to All Tenants=True),
        if id is given, query will be fillter alarms by project id.
        :param is_admin_project: True to return alarms with
        project_id=None or resource_project_id=None
        """

        session = self._engine_facade.get_session()
        query = session.query(models.Alarm)
        query = self._add_project_filtering_to_query(
            query, project_id, is_admin_project)

        self.assert_args(start, end, filter_by, filter_vals,
                         only_active_alarms, sort_dirs)

        if only_active_alarms:
            query = query.filter(models.Alarm.end_timestamp > db_time())
        elif (start and end) or start:
            query = self._add_time_frame_to_query(query, start, end)

        query = self._add_filtering_to_query(query, filter_by, filter_vals)

        if limit:
            query = self._generate_alarms_paginate_query(query,
                                                         limit,
                                                         sort_by,
                                                         sort_dirs,
                                                         next_page,
                                                         marker)
        elif limit == 0:
            sort_dir_func = {
                ASC: sqlalchemy.asc,
                DESC: sqlalchemy.desc,
            }
            for i in range(len(sort_by)):
                query.order_by(sort_dir_func[sort_dirs[i]](
                    getattr(models.Alarm, sort_by[i])))
        return query.all()

    @staticmethod
    def assert_args(start,
                    end,
                    filter_by,
                    filter_vals,
                    only_active_alarms,
                    sort_dirs):
        if only_active_alarms and (start or end):
            raise VitrageInputError("'only_active_alarms' can't be used "
                                    "with 'start' or 'end' ")
        if end and not start:
            raise VitrageInputError("'end' can't be used without 'start'")
        if (filter_by and not filter_vals) or (filter_vals and not filter_by):
            raise VitrageInputError('Cannot perform filtering, one of '
                                    'filter_by or filter_vals are missing')
        if filter_by and filter_vals and len(filter_by) != len(filter_vals):
            raise VitrageInputError("Cannot perform filtering, len of "
                                    "'filter_by' and 'filter_vals' differs")
        for d in sort_dirs:
            if d not in (ASC, DESC):
                raise VitrageInputError("Unknown sort direction %s" % d)

    @staticmethod
    def _add_time_frame_to_query(query, start, end):
        start = timeutils.normalize_time(start)
        if start and end:
            end = timeutils.normalize_time(end)
            query = \
                query.filter(
                    or_(and_(models.Alarm.start_timestamp >= start,
                             models.Alarm.start_timestamp <= end),
                        and_(models.Alarm.end_timestamp >= start,
                             models.Alarm.end_timestamp <= end),
                        and_(models.Alarm.start_timestamp <= start,
                             models.Alarm.end_timestamp >= end)))
        elif start:
            query = query.filter(models.Alarm.end_timestamp >= start)
        return query

    @staticmethod
    def _add_project_filtering_to_query(query, project_id=None,
                                        is_admin_project=False):

        if project_id:
            if is_admin_project:
                query = query.filter(or_(
                    or_(models.Alarm.project_id == project_id,
                        models.Alarm.vitrage_resource_project_id ==
                        project_id),
                    and_(
                        or_(
                            models.Alarm.project_id == project_id,
                            models.Alarm.project_id == None),
                        or_(
                            models.Alarm.vitrage_resource_project_id ==
                            project_id,
                            models.Alarm.vitrage_resource_project_id == None)
                    )))  # noqa
            else:
                query = query.filter(
                    or_(models.Alarm.project_id == project_id,
                        models.Alarm.vitrage_resource_project_id ==
                        project_id))
        return query

    @staticmethod
    def _add_filtering_to_query(query, filter_by, filter_vals):

        if not (filter_by or filter_vals):
            return query

        for i in range(len(filter_by)):
            key = filter_by[i]
            val = filter_vals[i]
            val = val if val and type(val) == list else [val]
            cond = or_(*[getattr(models.Alarm, key).like(
                '%' + val[j] + '%') for j in range(len(val))])
            query = query.filter(cond)
        return query

    def _generate_alarms_paginate_query(self,
                                        query,
                                        limit,
                                        sort_by,
                                        sort_dirs,
                                        next_page,
                                        marker):

        limit = min(int(limit), LIMIT)

        if marker:
            session = self._engine_facade.get_session()
            marker = session.query(models.Alarm). \
                filter(models.Alarm.vitrage_id ==
                       marker).first()

        if HProps.VITRAGE_ID not in sort_by:
            sort_by.append(HProps.VITRAGE_ID)
            sort_dirs.append(ASC)

        if not next_page and marker:  # 'not next_page' means previous page
            marker = self._create_marker_for_prev(
                query, limit, sort_by, sort_dirs, marker)

        query = sqlalchemyutils.paginate_query(query,
                                               models.Alarm,
                                               limit,
                                               sort_by,
                                               sort_dirs=sort_dirs,
                                               marker=marker)
        return query

    @staticmethod
    def _create_marker_for_prev(query, limit, sort_by, sort_dirs, marker):

        dirs = [DESC if d == ASC else ASC for d in sort_dirs]
        query = sqlalchemyutils.paginate_query(query,
                                               models.Alarm,
                                               limit + 1,
                                               sort_by,
                                               marker=marker,
                                               sort_dirs=dirs)

        alarms = query.all()
        if len(alarms) < limit + 1:
            new_marker = None
        else:
            new_marker = alarms[-1]

        return new_marker

    def alarm_rca(self,
                  alarm_id,
                  forward=True,
                  backward=True,
                  depth=None,
                  project_id=None,
                  admin=False):

        n_result_f = []
        e_result_f = []
        if forward:
            n_result_f, e_result_f = \
                self._bfs(alarm_id, self._out_rca, depth, admin=admin,
                          project_id=project_id)

        n_result_b = []
        e_result_b = []
        if backward:
            n_result_b, e_result_b = \
                self._bfs(alarm_id, self._in_rca, depth, admin=admin,
                          project_id=project_id)

        n_result = self.get_alarms(limit=0,
                                   filter_by=[HProps.VITRAGE_ID],
                                   filter_vals=[n_result_f + n_result_b])

        e_result = e_result_f + e_result_b

        return n_result, e_result

    def _rca_edges(self, filter_by, a_ids, proj_id, admin):
        alarm_ids = [str(alarm) for alarm in a_ids]
        session = self._engine_facade.get_session()
        query = session.query(models.Edge)\
            .filter(and_(getattr(models.Edge, filter_by).in_(alarm_ids),
                         models.Edge.label == ELabel.CAUSES))

        query = query.join(models.Edge.target)
        query = self._add_project_filtering_to_query(query, proj_id, admin)

        return query.all()

    def _out_rca(self, sources, proj_id, admin):
        return self._rca_edges(HProps.SOURCE_ID, sources, proj_id, admin)

    def _in_rca(self, targets, proj_id, admin):
        return self._rca_edges(HProps.TARGET_ID, targets, proj_id, admin)

    def _bfs(self, alarm_id, neighbors_func,
             depth=None,
             project_id=None,
             admin=False):
        n_result = []
        visited_nodes = set()
        n_result.append(alarm_id)
        e_result = []
        curr_depth = 0
        nodes_q = {curr_depth: [alarm_id]}
        while nodes_q:
            node_ids = nodes_q.pop(curr_depth)
            if depth and curr_depth >= depth:
                break
            for node_id in node_ids:
                if node_id in visited_nodes:
                    node_ids.remove(node_id)
            visited_nodes.update(node_ids)
            e_list = neighbors_func(node_ids, project_id, admin)
            n_list = \
                [edge.target_id if edge.source_id in node_ids
                 else edge.source_id for edge in e_list]
            n_result.extend(n_list)
            e_result.extend(e_list)
            if n_list:
                curr_depth += 1
                nodes_q[curr_depth] = n_list

        return n_result, e_result



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from vitrage.common.constants import TemplateTypes
from vitrage.evaluator.template_fields import TemplateFields
from vitrage.evaluator.template_validation.content.base import \
    get_content_correct_result
from vitrage.evaluator.template_validation.content.base import \
    get_content_fault_result
from vitrage.evaluator.template_validation.status_messages import status_msgs

LOG = log.getLogger(__name__)


class MetadataValidator(object):

    @classmethod
    def validate(cls, metadata):
        if not metadata:
            return get_content_fault_result(62)

        type = metadata.get(TemplateFields.TYPE)

        if not type:
            LOG.error('%s status code: %s' % (status_msgs[64], 64))
            return get_content_fault_result(64)

        if type not in TemplateTypes.types():
            LOG.error('%s status code: %s' % (status_msgs[65], 65))
            return get_content_fault_result(65)

        return get_content_correct_result()



# Copyright 2019 - China Mobile (SuZhou) Software Technology Co.,Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class HostProperties(object):
    HOST = 'host'
    ZONE = 'zone'


class ZoneProperties(object):
    ZONE_NAME = 'zoneName'
    ZONE_STATE = 'zoneState'
    NOVE_COMPUTE = 'nova-compute'
    ACTIVE = 'active'



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.tests.functional.base import TestFunctionalBase


class TestDataSourcesBase(TestFunctionalBase):

    def _find_entity_id_by_type(self, graph, vitrage_type):
        entity_vertices = graph.get_vertices(vertex_attr_filter={
            VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
            VProps.VITRAGE_TYPE: vitrage_type
        })
        self.assertGreater(len(entity_vertices), 0)

        return entity_vertices[0][VProps.ID]



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from vitrage.common.constants import EdgeProperties as EProps
from vitrage.common.constants import VertexProperties as VProps
from vitrage.graph import Edge
from vitrage.graph import Vertex
from vitrage.utils.datetime import utcnow


LOG = log.getLogger(__name__)


def is_newer_vertex(prev_vertex, new_vertex):
    prev_timestamp = prev_vertex.get(VProps.VITRAGE_SAMPLE_TIMESTAMP)
    if not prev_timestamp:
        return True

    new_timestamp = new_vertex.get(VProps.VITRAGE_SAMPLE_TIMESTAMP)
    if not new_timestamp:
        return True

    return prev_timestamp <= new_timestamp


def is_deleted(item):
    return item and \
        (isinstance(item, Vertex) and
         item.get(VProps.VITRAGE_IS_DELETED, False)) or \
        (isinstance(item, Edge) and
         item.get(EProps.VITRAGE_IS_DELETED, False))


def mark_deleted(g, item):
    if isinstance(item, Vertex):
        if item.get(VProps.VITRAGE_IS_DELETED, False):
            return
        item[VProps.VITRAGE_IS_DELETED] = True
        item[VProps.VITRAGE_SAMPLE_TIMESTAMP] = str(utcnow())
        g.update_vertex(item)
    elif isinstance(item, Edge):
        if item.get(EProps.VITRAGE_IS_DELETED, False):
            return
        item[EProps.VITRAGE_IS_DELETED] = True
        item[EProps.UPDATE_TIMESTAMP] = str(utcnow())
        g.update_edge(item)


def delete_placeholder_vertex(g, vertex):
    """Checks if it is a placeholder vertex, and if so deletes it """

    LOG.debug('Asked to delete a placeholder vertex: %s with %d neighbors',
              vertex, len(g.get_edges(vertex.vertex_id)))

    if not vertex[VProps.VITRAGE_IS_PLACEHOLDER]:
        return
    if not any(True for neighbor_edge in g.get_edges(vertex.vertex_id)
               if not is_deleted(neighbor_edge)):
        LOG.debug("Delete placeholder vertex: %s", vertex)
        g.remove_vertex(vertex)


def find_neighbor_types(neighbors):
    """Finds all the types (TYPE, SUB_TYPE) of the neighbors """

    neighbor_types = set()
    for (vertex, edge) in neighbors:
        neighbor_types.add(get_vertex_types(vertex))
    return neighbor_types


def get_vertex_types(vertex):
    vitrage_category = vertex.get(VProps.VITRAGE_CATEGORY)
    vitrage_type = vertex.get(VProps.VITRAGE_TYPE)
    if not vitrage_category:
        LOG.warning('no vitrage_category in vertex: %s', vertex)
    return vitrage_category, vitrage_type


def update_entity_graph_vertex(g, graph_vertex, updated_vertex):
    if updated_vertex[VProps.VITRAGE_IS_PLACEHOLDER] and \
            graph_vertex and not graph_vertex[VProps.VITRAGE_IS_PLACEHOLDER]:

        updated_vertex[VProps.VITRAGE_IS_PLACEHOLDER] = False
        updated_vertex[VProps.VITRAGE_IS_DELETED] = \
            graph_vertex[VProps.VITRAGE_IS_DELETED]

    g.update_vertex(updated_vertex,
                    not updated_vertex[VProps.VITRAGE_IS_PLACEHOLDER])



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import queue

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceProperties as DSProp
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import TemplateTopologyFields as TTFields
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.alarm_properties import AlarmProperties as AlarmProps
from vitrage.datasources.nagios import NAGIOS_DATASOURCE
from vitrage.datasources.nagios.properties import NagiosProperties as NProps
from vitrage.datasources.nagios.properties import NagiosTestStatus
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.entity_graph.mappings.operational_alarm_severity import \
    OperationalAlarmSeverity
from vitrage.entity_graph.mappings.operational_resource_state import \
    OperationalResourceState
from vitrage.evaluator.actions.action_executor import ActionExecutor
from vitrage.evaluator.actions.base import ActionMode
from vitrage.evaluator.actions.base import ActionType
from vitrage.evaluator.actions.evaluator_event_transformer \
    import VITRAGE_DATASOURCE
from vitrage.evaluator.actions.recipes.action_steps import ADD_VERTEX
from vitrage.evaluator.actions.recipes.base import EVALUATOR_EVENT_TYPE
from vitrage.evaluator.scenario_evaluator import ActionInfo
from vitrage.evaluator.template_data import ActionSpecs
from vitrage.evaluator.template_fields import TemplateFields as TFields
from vitrage.opts import register_opts
from vitrage.tests.functional.base import TestFunctionalBase
from vitrage.tests.functional.test_configuration import TestConfiguration


class TestActionExecutor(TestFunctionalBase, TestConfiguration):

    def setUp(self):
        super(TestActionExecutor, self).setUp()
        self.add_db()

        for datasource_name in self.conf.datasources.types:
            register_opts(datasource_name, self.conf.datasources.path)

    def _init_executer(self):
        event_queue = queue.Queue()

        def actions_callback(event_type, data):
            event_queue.put(data)

        return event_queue, ActionExecutor(actions_callback)

    def test_execute_set_state(self):

        # Test Setup
        processor = self._create_processor_with_graph()

        vertex_attrs = {VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE}
        host_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=vertex_attrs)
        host_vertex_before = host_vertices[0]

        targets = {TFields.TARGET: host_vertex_before}
        props = {TFields.STATE: OperationalResourceState.SUBOPTIMAL}
        action_spec = ActionSpecs(0, ActionType.SET_STATE, targets, props)

        event_queue, action_executor = self._init_executer()

        # Test Action - do
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.DO, None, None)])
        self._consume_queue(event_queue, processor)

        host_vertex_after = processor.entity_graph.get_vertex(
            host_vertex_before.vertex_id)

        # Test Assertions
        agg_state_before = \
            host_vertex_before.get(VProps.VITRAGE_AGGREGATED_STATE)
        self.assertNotEqual(agg_state_before,
                            OperationalResourceState.SUBOPTIMAL)
        self.assertNotIn(VProps.VITRAGE_STATE, host_vertex_before.properties)

        agg_state_after = \
            host_vertex_after.get(VProps.VITRAGE_AGGREGATED_STATE)
        self.assertEqual(agg_state_after, OperationalResourceState.SUBOPTIMAL)
        v_state_after = host_vertex_after.get(VProps.VITRAGE_STATE)
        self.assertEqual(v_state_after, OperationalResourceState.SUBOPTIMAL)

        # Test Action - undo
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.UNDO, None, None)])
        self._consume_queue(event_queue, processor)

        host_vertex_after_undo = processor.entity_graph.get_vertex(
            host_vertex_before.vertex_id)

        # Test Assertions
        agg_state_after_undo = \
            host_vertex_before.get(VProps.VITRAGE_AGGREGATED_STATE)
        self.assertEqual(agg_state_after_undo, agg_state_before)
        self.assertNotIn(
            VProps.VITRAGE_STATE, host_vertex_after_undo.properties)

    def test_execute_mark_instance_down(self):

        # Test Setup
        processor = self._create_processor_with_graph()

        vertex_attrs = {VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE}
        instance_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=vertex_attrs)
        instance_vertex_before = instance_vertices[0]

        targets = {TFields.TARGET: instance_vertex_before}
        props = {}
        action_spec = ActionSpecs(0, ActionType.MARK_DOWN, targets, props)

        event_queue, action_executor = self._init_executer()

        # Test Action - do
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.DO, None, None)])
        self._consume_queue(event_queue, processor)

        instance_vertex_after = processor.entity_graph.get_vertex(
            instance_vertex_before.vertex_id)

        # Test Assertions
        self.assertTrue(instance_vertex_after.get(VProps.IS_MARKED_DOWN))

        # Test Action - undo
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.UNDO, None, None)])
        self._consume_queue(event_queue, processor)

        instance_vertex_after_undo = processor.entity_graph.get_vertex(
            instance_vertex_before.vertex_id)

        # Test Assertions
        self.assertFalse(instance_vertex_after_undo.get(VProps.IS_MARKED_DOWN))

    def test_execute_mark_down(self):

        # Test Setup
        processor = self._create_processor_with_graph()

        vertex_attrs = {VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE}
        host_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=vertex_attrs)
        host_vertex_before = host_vertices[0]

        targets = {TFields.TARGET: host_vertex_before}
        props = {}
        action_spec = ActionSpecs(0, ActionType.MARK_DOWN, targets, props)

        event_queue, action_executor = self._init_executer()

        # Test Action - do
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.DO, None, None)])
        self._consume_queue(event_queue, processor)

        host_vertex_after = processor.entity_graph.get_vertex(
            host_vertex_before.vertex_id)

        # Test Assertions
        self.assertTrue(host_vertex_after.get(VProps.IS_MARKED_DOWN))

        # Test Action - undo
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.UNDO, None, None)])
        self._consume_queue(event_queue, processor)

        host_vertex_after_undo = processor.entity_graph.get_vertex(
            host_vertex_before.vertex_id)

        # Test Assertions
        self.assertFalse(host_vertex_after_undo.get(VProps.IS_MARKED_DOWN))

    def test_execute_add_edge(self):

        # Test Setup
        processor = self._create_processor_with_graph()

        vertex_attrs = {VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE}
        host_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=vertex_attrs)

        host_1 = host_vertices[0]
        nagios_event1 = TestActionExecutor._get_nagios_event(
            host_1.get(VProps.ID), NOVA_HOST_DATASOURCE)
        processor.process_event(nagios_event1)

        host_2 = host_vertices[1]
        nagios_event2 = TestActionExecutor._get_nagios_event(
            host_2.get(VProps.ID), NOVA_HOST_DATASOURCE)
        processor.process_event(nagios_event2)

        alarms_attrs = {VProps.VITRAGE_TYPE: NAGIOS_DATASOURCE}
        alarms_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=alarms_attrs)

        alarm1 = alarms_vertices[0]
        alarm2 = alarms_vertices[1]
        targets = {
            TFields.TARGET: alarm1,
            TFields.SOURCE: alarm2
        }
        action_spec = ActionSpecs(
            0, ActionType.ADD_CAUSAL_RELATIONSHIP, targets, {})

        event_queue, action_executor = self._init_executer()

        before_edge = processor.entity_graph.get_edge(alarm2.vertex_id,
                                                      alarm1.vertex_id,
                                                      EdgeLabel.CAUSES)
        # Test Action - do
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.DO, None, None)])
        self._consume_queue(event_queue, processor)

        new_edge = processor.entity_graph.get_edge(alarm2.vertex_id,
                                                   alarm1.vertex_id,
                                                   EdgeLabel.CAUSES)
        # Test Assertions
        self.assertIsNone(before_edge)
        self.assertIsNotNone(new_edge)

    def test_execute_add_vertex(self):

        # Test Setup
        processor = self._create_processor_with_graph()

        vertex_attrs = {VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE}
        host_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=vertex_attrs)

        host = host_vertices[0]

        targets = {TFields.TARGET: host}
        props = {
            TFields.ALARM_NAME: 'VM_CPU_SUBOPTIMAL_PERFORMANCE',
            TFields.SEVERITY: OperationalAlarmSeverity.CRITICAL,
            VProps.STATE: AlarmProps.ACTIVE_STATE,
            VProps.RESOURCE_ID: host[VProps.ID],
            VProps.VITRAGE_ID: 'DUMMY_ID'
        }

        # Raise alarm action adds new vertex with type vitrage to the graph
        action_spec = ActionSpecs(0, ActionType.RAISE_ALARM, targets, props)

        alarm_vertex_attrs = {VProps.VITRAGE_TYPE: VITRAGE_DATASOURCE}
        before_alarms = processor.entity_graph.get_vertices(
            vertex_attr_filter=alarm_vertex_attrs)

        event_queue, action_executor = self._init_executer()

        # Test Action
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.DO, None, None)])
        self._consume_queue(event_queue, processor)

        after_alarms = processor.entity_graph.get_vertices(
            vertex_attr_filter=alarm_vertex_attrs)

        # Assertions
        self.assertEqual(len(before_alarms) + 1, len(after_alarms))
        self.assert_is_not_empty(after_alarms)

        alarm = after_alarms[0]

        self.assertEqual(alarm.properties[VProps.VITRAGE_CATEGORY],
                         EntityCategory.ALARM)
        self.assertEqual(alarm.properties[VProps.VITRAGE_TYPE],
                         VITRAGE_DATASOURCE)
        self.assertEqual(alarm.properties[VProps.SEVERITY],
                         props[TFields.SEVERITY])
        self.assertEqual(alarm.properties[VProps.VITRAGE_OPERATIONAL_SEVERITY],
                         props[TFields.SEVERITY])
        self.assertEqual(alarm.properties[VProps.STATE],
                         AlarmProps.ACTIVE_STATE)
        self.assertEqual(alarm.properties[VProps.VITRAGE_RESOURCE_ID],
                         action_spec.targets
                         [TTFields.TARGET][VProps.VITRAGE_ID]),
        self.assertEqual(alarm.properties[VProps.VITRAGE_RESOURCE_TYPE],
                         NOVA_HOST_DATASOURCE)

    def test_execute_add_and_remove_vertex(self):

        # Test Setup
        processor = self._create_processor_with_graph()

        vertex_attrs = {VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE}
        host_vertices = processor.entity_graph.get_vertices(
            vertex_attr_filter=vertex_attrs)

        host = host_vertices[0]

        targets = {TFields.TARGET: host}
        props = {
            TFields.ALARM_NAME: 'VM_CPU_SUBOPTIMAL_PERFORMANCE',
            TFields.SEVERITY: OperationalAlarmSeverity.CRITICAL,
            VProps.STATE: AlarmProps.ACTIVE_STATE,
            VProps.RESOURCE_ID: host[VProps.ID]
        }
        action_spec = ActionSpecs(0, ActionType.RAISE_ALARM, targets, props)

        add_vertex_event = TestActionExecutor._get_vitrage_add_vertex_event(
            host,
            props[TFields.ALARM_NAME],
            props[TFields.SEVERITY])

        processor.process_event(add_vertex_event)

        alarm_vertex_attrs = {VProps.VITRAGE_TYPE: VITRAGE_DATASOURCE,
                              VProps.VITRAGE_IS_DELETED: False}
        before_alarms = processor.entity_graph.get_vertices(
            vertex_attr_filter=alarm_vertex_attrs)

        event_queue, action_executor = self._init_executer()

        # Test Action - undo
        action_executor.execute(
            [ActionInfo(action_spec, ActionMode.UNDO, None, None)])
        self._consume_queue(event_queue, processor)

        after_alarms = processor.entity_graph.get_vertices(
            vertex_attr_filter=alarm_vertex_attrs)

        # Test Assertions
        self.assertEqual(len(before_alarms) - 1, len(after_alarms))

    @staticmethod
    def _get_nagios_event(resource_name, resource_type):

        return {NProps.LAST_CHECK: '2016-02-07 15:26:04',
                NProps.RESOURCE_NAME: resource_name,
                NProps.RESOURCE_TYPE: resource_type,
                NProps.SERVICE: 'Check_MK',
                NProps.STATUS: NagiosTestStatus.CRITICAL,
                NProps.STATUS_INFO: 'test test test',
                DSProp.DATASOURCE_ACTION: DatasourceAction.SNAPSHOT,
                DSProp.ENTITY_TYPE: NAGIOS_DATASOURCE,
                DSProp.SAMPLE_DATE: '2016-02-07T15:26:04Z'}

    @staticmethod
    def _get_vitrage_add_vertex_event(target_vertex, alarm_name, severity):

        return {TTFields.TARGET: target_vertex.vertex_id,
                VProps.UPDATE_TIMESTAMP: '2016-03-17 11:33:32.443002',
                DSProp.DATASOURCE_ACTION: DatasourceAction.UPDATE,
                TFields.ALARM_NAME: alarm_name,
                VProps.STATE: 'Active',
                EVALUATOR_EVENT_TYPE: ADD_VERTEX,
                DSProp.ENTITY_TYPE: VITRAGE_DATASOURCE,
                VProps.SEVERITY: OperationalAlarmSeverity.CRITICAL,
                VProps.VITRAGE_ID: 'mock_vitrage_id',
                VProps.VITRAGE_RESOURCE_TYPE: NOVA_HOST_DATASOURCE,
                VProps.VITRAGE_CATEGORY: EntityCategory.ALARM,
                VProps.VITRAGE_SAMPLE_TIMESTAMP:
                    '2016-03-17 11:33:32.443002+00:00'}



# Copyright 2016 - Nokia Corporation
# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from datetime import datetime
# noinspection PyPackageRequirements
from unittest import mock
from vitrage.common.utils import compress_obj

from vitrage.storage.sqlalchemy import models
from vitrage.tests.functional.api.v1 import FunctionalTest


class NoAuthTest(FunctionalTest):

    def __init__(self, *args, **kwds):
        super(NoAuthTest, self).__init__(*args, **kwds)
        self.auth = 'noauth'

    def test_noauth_mode_post_event(self):

        with mock.patch('pecan.request') as request:

            details = {
                'hostname': 'host123',
                'source': 'sample_monitor',
                'cause': 'another alarm',
                'severity': 'critical',
                'status': 'down',
                'monitor_id': 'sample monitor',
                'monitor_event_id': '456',
            }
            event_time = datetime.now().isoformat()
            event_type = 'compute.host.down'

            resp = self.post_json('/event/', params={'time': event_time,
                                                     'type': event_type,
                                                     'details': details})

            self.assertEqual(1, request.client.call.call_count)
            self.assertEqual('200 OK', resp.status)

    def test_noauth_mode_get_topology(self):
        with mock.patch('pecan.request') as request:
            request.client.call.return_value = compress_obj({})
            params = dict(depth=None, graph_type='graph', query=None,
                          root=None,
                          all_tenants=False)
            resp = self.post_json('/topology/', params=params)

            self.assertEqual(1, request.client.call.call_count)
            self.assertEqual('200 OK', resp.status)
            self.assert_is_empty(resp.json)

    def test_noauth_mode_list_alarms(self):
        with mock.patch('pecan.request') as request:
            request.client.call.return_value = compress_obj({"alarms": []})
            params = dict(vitrage_id='all', all_tenants=False)
            data = self.get_json('/alarm/', params=params)

            self.assertEqual(1, request.client.call.call_count)
            self.assert_is_empty(data)

    def test_noauth_mode_show_alarm(self):

        with mock.patch('pecan.request') as request:
            request.client.call.return_value = '{}'
            data = self.get_json('/alarm/1234')

            self.assertEqual(1, request.client.call.call_count)
            self.assert_is_empty(data)

    def test_noauth_mode_show_alarm_count(self):
        with mock.patch('pecan.request') as request:
            request.client.call.return_value = '{}'
            params = dict(all_tenants=False)
            resp = self.post_json('/alarm/count/', params=params)

            self.assertEqual(1, request.client.call.call_count)
            self.assertEqual('200 OK', resp.status)
            self.assert_is_empty(resp.json)

    def test_noauth_mode_list_resources(self):

        with mock.patch('pecan.request') as request:
            request.client.call.return_value = compress_obj({"resources": []})
            params = dict(resource_type='all', all_tenants=False)
            data = self.get_json('/resources/', params=params)

            self.assertEqual(1, request.client.call.call_count)
            self.assert_is_empty(data)

    def test_noauth_mode_show_resource(self):

        with mock.patch('pecan.request') as request:
            request.client.call.return_value = '{}'
            data = self.get_json('/resources/1234')

            self.assertEqual(1, request.client.call.call_count)
            self.assert_is_empty(data)

    def test_noauth_mode_list_templates(self):

        with mock.patch('pecan.request') as request:
            request.storage.templates.query.return_value = []
            data = self.get_json('/template/')

            self.assertEqual(1, request.storage.templates.query.call_count)
            self.assert_is_empty(data)

    def test_noauth_mode_show_template(self):

        with mock.patch('pecan.request') as request:
            request.storage.templates.query.return_value = \
                [models.Template(file_content={})]
            data = self.get_json('/template/1234')

            self.assertEqual(1, request.storage.templates.query.call_count)
            self.assertEqual(
                1, request.storage.templates.query_with_status_not.call_count)
            self.assert_is_empty(data)

    def test_noauth_mode_validate_template(self):

        with mock.patch('pecan.request') as request:
            request.client.call.return_value = '{}'
            params = {"templates": {}}
            resp = self.post_json('/template/', params=params)

            self.assertEqual(1, request.client.call.call_count)
            self.assertEqual('200 OK', resp.status)
            self.assert_is_empty(resp.json)

    def test_noauth_mode_get_rca(self):

        with mock.patch('pecan.request') as request:
            request.client.call.return_value = '{}'
            params = dict(all_tenants=False)
            data = self.get_json('/rca/1234/', params=params)

            self.assertEqual(1, request.client.call.call_count)
            self.assert_is_empty(data)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from testtools import matchers

from vitrage.common.constants import VertexProperties as VProps
from vitrage.evaluator.actions.base import ActionType
from vitrage.evaluator.actions.recipes.action_steps import UPDATE_VERTEX
from vitrage.evaluator.actions.recipes.mark_down import MarkDown
from vitrage.evaluator.template_data import ActionSpecs
from vitrage.evaluator.template_fields import TemplateFields as TFields
from vitrage.graph import Vertex
from vitrage.tests import base


class MarkDownRecipeTest(base.BaseTest):

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(MarkDownRecipeTest, cls).setUpClass()
        cls.target_vertex = Vertex('RESOURCE:nova.host:test1')

        targets = {TFields.TARGET: cls.target_vertex}
        cls.action_spec = ActionSpecs(0, ActionType.MARK_DOWN, targets, None)

    def test_get_do_recipe(self):

        # Test Action
        action_steps = MarkDown.get_do_recipe(self.action_spec)

        # Test Assertions

        # expecting for one step: [update_vertex]
        self.assertThat(action_steps, matchers.HasLength(1))

        self.assertEqual(UPDATE_VERTEX, action_steps[0].type)
        update_vertex_step_params = action_steps[0].params
        self.assertThat(update_vertex_step_params, matchers.HasLength(3))

        is_marked_down = update_vertex_step_params[VProps.IS_MARKED_DOWN]
        self.assertTrue(is_marked_down)

        vitrage_id = update_vertex_step_params[VProps.VITRAGE_ID]
        self.assertEqual(self.target_vertex.vertex_id, vitrage_id)

        is_real_vitrage_id = \
            update_vertex_step_params[VProps.IS_REAL_VITRAGE_ID]
        self.assertTrue(is_real_vitrage_id)

    def test_get_undo_recipe(self):

        # Test Action
        action_steps = MarkDown.get_undo_recipe(self.action_spec)

        # Test Assertions

        # expecting for one step: [update_vertex]
        self.assertThat(action_steps, matchers.HasLength(1))

        self.assertEqual(UPDATE_VERTEX, action_steps[0].type)
        update_vertex_step_params = action_steps[0].params
        self.assertThat(update_vertex_step_params, matchers.HasLength(3))

        is_marked_down = update_vertex_step_params[VProps.IS_MARKED_DOWN]
        self.assertFalse(is_marked_down)

        vitrage_id = update_vertex_step_params[VProps.VITRAGE_ID]
        self.assertEqual(self.target_vertex.vertex_id, vitrage_id)

        is_real_vitrage_id = \
            update_vertex_step_params[VProps.IS_REAL_VITRAGE_ID]
        self.assertTrue(is_real_vitrage_id)



# Copyright 2017 - ZTE
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from datetime import datetime

from oslo_config import cfg
from oslo_log import log
import oslo_messaging
from oslo_utils import uuidutils
from pyasn1.codec.ber import decoder
from pysnmp.carrier.asyncore.dgram import udp
from pysnmp.carrier.asyncore.dgram import udp6
from pysnmp.carrier.asyncore.dispatch import AsyncoreDispatcher
from pysnmp.proto import api as snmp_api
from pysnmp.proto.rfc1902 import Integer
import sys

from vitrage.common.constants import EventProperties
from vitrage.coordination import service as coord
from vitrage.datasources.transformer_base import extract_field_value
from vitrage.messaging import get_transport
from vitrage.snmp_parsing.properties import SnmpEventProperties as SEProps
from vitrage.utils.file import load_yaml_file

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class SnmpParsingService(coord.Service):
    RUN_FOREVER = 1

    def __init__(self, worker_id):
        super(SnmpParsingService, self).__init__(worker_id)
        self.listening_port = CONF.snmp_parsing.snmp_listening_port
        self.oid_mapping = \
            load_yaml_file(CONF.snmp_parsing.oid_mapping)
        self._init_oslo_notifier()

    def run(self):
        super(SnmpParsingService, self).run()
        LOG.info("Vitrage SNMP Parsing Service - Starting...")

        transport_dispatcher = AsyncoreDispatcher()
        transport_dispatcher.registerRecvCbFun(self.callback_func)

        trans_udp = udp.UdpSocketTransport()
        udp_transport = \
            trans_udp.openServerMode(('0.0.0.0', self.listening_port))

        trans_udp6 = udp6.Udp6SocketTransport()
        udp6_transport = \
            trans_udp6.openServerMode(('::1', self.listening_port))

        transport_dispatcher.registerTransport(udp.domainName, udp_transport)
        transport_dispatcher.registerTransport(udp6.domainName, udp6_transport)
        LOG.info("Vitrage SNMP Parsing Service - Started!")

        transport_dispatcher.jobStarted(self.RUN_FOREVER)
        try:
            transport_dispatcher.runDispatcher()
        except Exception:
            LOG.error("Run transport dispatcher failed.")
            transport_dispatcher.closeDispatcher()
            raise

    def terminate(self):
        super(SnmpParsingService, self).terminate()
        LOG.info("Vitrage SNMP Parsing Service - Stopping...")
        LOG.info("Vitrage SNMP Parsing Service - Stopped!")

    # noinspection PyUnusedLocal
    def callback_func(self, transport_dispatcher, transport_domain,
                      transport_address, whole_msg):
        while whole_msg:
            msg_ver = int(snmp_api.decodeMessageVersion(whole_msg))
            if msg_ver in snmp_api.protoModules:
                p_mod = snmp_api.protoModules[msg_ver]
            else:
                LOG.error('Unsupported SNMP version %s.' % msg_ver)
                return
            req_msg, whole_msg = decoder.decode(
                whole_msg, asn1Spec=p_mod.Message(),
            )
            req_pdu = p_mod.apiMessage.getPDU(req_msg)
            if req_pdu.isSameTypeWith(p_mod.TrapPDU()):
                ver_binds = p_mod.apiTrapPDU.getVarBinds(req_pdu) \
                    if msg_ver == snmp_api.protoVersion1 \
                    else p_mod.apiPDU.getVarBinds(req_pdu)

                binds_dict = self._convert_binds_to_dict(ver_binds)
                LOG.debug('Receive binds info after convert: %s' % binds_dict)
                self._send_snmp_to_queue(binds_dict)

    def _convert_binds_to_dict(self, var_binds):
        binds_dict = {}
        for oid, val in var_binds:
            u_oid = self._convert_obj_to_unicode(oid)
            binds_dict[u_oid] = int(val) if isinstance(val, Integer) \
                else self._convert_obj_to_unicode(val)
        return binds_dict

    @staticmethod
    def _convert_obj_to_unicode(val):
        if sys.version_info[0] < 3:
            return str(val).decode('iso-8859-1')
        return str(val)

    def _init_oslo_notifier(self):
        self.oslo_notifier = None
        try:
            self.publisher = 'vitrage-snmp-parsing'
            self.oslo_notifier = oslo_messaging.Notifier(
                get_transport(),
                driver='messagingv2',
                publisher_id=self.publisher,
                topics=['vitrage_notifications'])
        except Exception:
            LOG.exception('Failed to initialize oslo notifier')

    def _send_snmp_to_queue(self, snmp_trap):
        try:
            event_type = self._get_event_type(snmp_trap)
            if not event_type:
                return
            event = {EventProperties.TIME: datetime.utcnow(),
                     EventProperties.TYPE: event_type,
                     EventProperties.DETAILS: snmp_trap}
            LOG.debug('snmp oslo_notifier event: %s' % event)
            self.oslo_notifier.info(
                ctxt={'message_id': uuidutils.generate_uuid(),
                      'publisher_id': self.publisher,
                      'timestamp': datetime.utcnow()},
                event_type=event_type,
                payload=event)
        except Exception as e:
            LOG.warning('Snmp failed to post event. Exception: %s', e)

    def _get_event_type(self, snmp_trap):
        if not self.oid_mapping:
            LOG.warning('No snmp trap is configured!')
            return None

        for mapping_info in self.oid_mapping:
            system_oid = extract_field_value(mapping_info, SEProps.SYSTEM_OID)
            conf_system = extract_field_value(mapping_info, SEProps.SYSTEM)
            if conf_system == extract_field_value(snmp_trap, system_oid):
                LOG.debug('snmp trap mapped the system: %s.' % conf_system)
                return extract_field_value(mapping_info, SEProps.EVENT_TYPE)

        LOG.error("Snmp trap does not contain system info!")
        return None



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy

from vitrage.evaluator.template_fields import TemplateFields
from vitrage.evaluator.template_validation.content import \
    template_content_validator as validator
from vitrage.tests.mocks import utils
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    ValidatorTest
from vitrage.utils import file as file_utils


CONDITION_TEMPLATES_DIR = '%s/templates/evaluator/conditions/%s'
REGEX_TEMPLATE_DIR = '%s/templates/regex/%s'
VERSION_TEMPLATE_DIR = '%s/templates/version/%s'


class TemplateContentValidatorTest(ValidatorTest):

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(TemplateContentValidatorTest, cls).setUpClass()

        template_dir_path = '%s/templates/general' % utils.get_resources_dir()
        cls.def_templates_tests_path = '%s/templates/def_template_tests/' % \
                                       utils.get_resources_dir()
        cls.def_templates_dir_path = cls.def_templates_tests_path +\
            'definition_templates'
        cls.templates = file_utils.load_yaml_files(template_dir_path)
        def_templates_list = file_utils.load_yaml_files(
            cls.def_templates_dir_path)
        cls.def_templates = utils.get_def_templates_dict_from_list(
            def_templates_list)
        cls.first_template = cls.templates[0]

        cls._hide_useless_logging_messages()

    def test_template_with_conflicting_include_entities(self):
        template_path = '/templates/with_conflicting_include_entities.yaml'
        template = file_utils.load_yaml_file(self.def_templates_tests_path +
                                             template_path)

        self._execute_and_assert_with_fault_result(template,
                                                   2,
                                                   self.def_templates)

    def test_template_with_no_defs_only_includes(self):
        template_path = '/templates/only_using_def_template_definitions.yaml'
        template = file_utils.load_yaml_file(self.def_templates_tests_path +
                                             template_path)

        self._execute_and_assert_with_correct_result(template,
                                                     self.def_templates)

    def test_template_with_multiple_includes(self):
        template_path = '/templates/basic_with_two_includes.yaml'
        template = file_utils.load_yaml_file(self.def_templates_tests_path +
                                             template_path)

        self._execute_and_assert_with_correct_result(template,
                                                     self.def_templates)

    def test_template_with_nonexisiting_includes(self):

        template_path = '/templates/basic_with_include_that_doesnt_exist.yaml'
        template = file_utils.load_yaml_file(self.def_templates_tests_path +
                                             template_path)
        self._execute_and_assert_with_fault_result(template, 142)

    def test_template_with_missing_def_template_dir(self):

        template_path = '/templates/basic_with_include.yaml'
        template = file_utils.load_yaml_file(self.def_templates_tests_path +
                                             template_path)
        self._execute_and_assert_with_fault_result(template, 142)

    def test_template_with_include(self):

        template_path = '/templates/basic_with_include.yaml'
        template = file_utils.load_yaml_file(self.def_templates_tests_path +
                                             template_path)
        self._execute_and_assert_with_correct_result(template,
                                                     self.def_templates)

    @property
    def clone_template(self):
        return copy.deepcopy(self.first_template)

    def test_template_validator(self):
        for template in self.templates:
            self._execute_and_assert_with_correct_result(template)

    def test_not_operator(self):
        basic_correct_not_condition_path = \
            '%s/templates/not_operator/basic_correct_not_condition.yaml' % \
            utils.get_resources_dir()
        basic_correct_not_condition_template = \
            file_utils.load_yaml_file(basic_correct_not_condition_path)
        self._execute_and_assert_with_correct_result(
            basic_correct_not_condition_template)

        basic_incorrect_not_condition_path = \
            '%s/templates/not_operator/basic_incorrect_not_condition.yaml' % \
            utils.get_resources_dir()
        basic_incorrect_not_condition_template = \
            file_utils.load_yaml_file(basic_incorrect_not_condition_path)
        self._execute_and_assert_with_fault_result(
            basic_incorrect_not_condition_template,
            86)

        complicated_correct_not_condition_path = \
            '%s/templates/not_operator/' \
            'complicated_correct_not_condition.yaml' % \
            utils.get_resources_dir()
        complicated_correct_not_condition_template = \
            file_utils.load_yaml_file(complicated_correct_not_condition_path)
        self._execute_and_assert_with_correct_result(
            complicated_correct_not_condition_template)

        complicated_incorrect_not_condition_path = \
            '%s/templates/not_operator/' \
            'complicated_incorrect_not_condition.yaml' % \
            utils.get_resources_dir()
        complicated_incorrect_not_condition_template = \
            file_utils.load_yaml_file(complicated_incorrect_not_condition_path)
        self._execute_and_assert_with_fault_result(
            complicated_incorrect_not_condition_template,
            86)

    def test_validate_entity_definition_with_no_unique_template_id(self):

        template = self.clone_template
        definitions = template[TemplateFields.DEFINITIONS]

        for entity in definitions[TemplateFields.ENTITIES]:
            entity_dict = entity[TemplateFields.ENTITY]
            entity_dict[TemplateFields.TEMPLATE_ID] = 'aaa'

        self._execute_and_assert_with_fault_result(template, 2)

    def test_validate_relationship_with_no_unique_template_id(self):

        template = self.clone_template
        definitions = template[TemplateFields.DEFINITIONS]
        entity = definitions[TemplateFields.ENTITIES][0]
        entity_id = entity[TemplateFields.ENTITY][TemplateFields.TEMPLATE_ID]
        relationship = definitions[TemplateFields.RELATIONSHIPS][0]
        relationship_dict = relationship[TemplateFields.RELATIONSHIP]
        relationship_dict[TemplateFields.TEMPLATE_ID] = entity_id

        self._execute_and_assert_with_fault_result(template, 2)

    def test_validate_relationship_with_invalid_target(self):

        template = self.clone_template
        definitions = template[TemplateFields.DEFINITIONS]
        relationship = definitions[TemplateFields.RELATIONSHIPS][0]
        relationship_dict = relationship[TemplateFields.RELATIONSHIP]
        relationship_dict[TemplateFields.TARGET] = 'unknown'

        self._execute_and_assert_with_fault_result(template, 3)

    def test_validate_relationship_with_invalid_source(self):

        template = self.clone_template
        definitions = template[TemplateFields.DEFINITIONS]
        relationship = definitions[TemplateFields.RELATIONSHIPS][0]
        relationship_dict = relationship[TemplateFields.RELATIONSHIP]
        relationship_dict[TemplateFields.SOURCE] = 'unknown'

        self._execute_and_assert_with_fault_result(template, 3)

    def test_validate_scenario_invalid_condition(self):

        template = self.clone_template
        scenario = template[TemplateFields.SCENARIOS][0]
        scenario_dict = scenario[TemplateFields.SCENARIO]

        scenario_dict[TemplateFields.CONDITION] = 'and resource'
        self._execute_and_assert_with_fault_result(template, 85)

        scenario_dict[TemplateFields.CONDITION] = 'resource or'
        self._execute_and_assert_with_fault_result(template, 85)

        scenario_dict[TemplateFields.CONDITION] = 'not or resource'
        self._execute_and_assert_with_fault_result(template, 85)

        scenario_dict[TemplateFields.CONDITION] = \
            'alarm_on_host (alarm or resource'
        self._execute_and_assert_with_fault_result(template, 85)

        scenario_dict[TemplateFields.CONDITION] = 'aaa'
        self._execute_and_assert_with_fault_result(template, 3)

        scenario_dict[TemplateFields.CONDITION] = 'resource and aaa'
        self._execute_and_assert_with_fault_result(template, 3)

    def test_validate_scenario_target_one_edge_condition(self):
        self._execute_condition_template_with_correct_result('one_edge.yaml')

    def test_validate_scenario_target_one_vertex_condition(self):
        self._execute_condition_template_with_correct_result('one_vertex.yaml')

    def test_validate_scenario_target_simple_or_condition(self):
        self._execute_condition_template_with_correct_result('simple_or.yaml')

    def test_validate_scenario_target_simple_or2_condition(self):
        self._execute_condition_template_with_correct_result('simple_or2.yaml')

    def test_validate_scenario_target_simple_or3_condition(self):
        self._execute_condition_template_with_correct_result('simple_or3.yaml')

    def test_validate_scenario_target_simple_or_unsupported_condition(self):
        self._execute_condition_template_with_fault_result(
            'simple_or_unsupported.yaml', 135)

    def test_validate_scenario_target_simple_and_condition(self):
        self._execute_condition_template_with_correct_result('simple_and.yaml')

    def test_validate_scenario_target_simple_and2_condition(self):
        self._execute_condition_template_with_correct_result(
            'simple_and2.yaml')

    def test_validate_scenario_target_complex1_condition(self):
        self._execute_condition_template_with_correct_result('complex1.yaml')

    def test_validate_scenario_target_complex2_condition(self):
        self._execute_condition_template_with_correct_result('complex2.yaml')

    def test_validate_scenario_target_not_edge_unsupported_condition(self):
        self._execute_condition_template_with_fault_result(
            'not_edge_unsupported.yaml', 134)

    def test_validate_scenario_target_not_or_unsupported__condition(self):
        self._execute_condition_template_with_fault_result(
            'not_or_unsupported.yaml', 134)

    def test_validate_scenario_target_not_or_unsupported2_condition(self):
        self._execute_condition_template_with_fault_result(
            'not_or_unsupported2.yaml', 135)

    def test_validate_scenario_target_complex_not_condition(self):
        self._execute_condition_template_with_correct_result(
            'complex_not.yaml')

    def test_validate_scenario_target_complex_not_unsupported_condition(self):
        self._execute_condition_template_with_fault_result(
            'complex_not_unsupported.yaml', 135)

    def test_faulty_regex(self):
        faulty_regex_path = \
            REGEX_TEMPLATE_DIR % (utils.get_resources_dir(),
                                  "faulty_regex.yaml")
        faulty_regex_template = \
            file_utils.load_yaml_file(faulty_regex_path)
        self._execute_and_assert_with_fault_result(
            faulty_regex_template, 47)

    def test_basic_regex(self):
        basic_regex_path = \
            REGEX_TEMPLATE_DIR % (utils.get_resources_dir(),
                                  "basic_regex.yaml")
        basic_regex_template = \
            file_utils.load_yaml_file(basic_regex_path)
        self._execute_and_assert_with_correct_result(
            basic_regex_template)

    def test_validate_template_with_no_version(self):
        invalid_version_path = \
            VERSION_TEMPLATE_DIR % (utils.get_resources_dir(),
                                    "no_version.yaml")
        template = file_utils.load_yaml_file(invalid_version_path)
        self._execute_and_assert_with_correct_result(template)

    def test_validate_template_with_version_1(self):
        invalid_version_path = \
            VERSION_TEMPLATE_DIR % (utils.get_resources_dir(),
                                    "v1/version1.yaml")
        template = file_utils.load_yaml_file(invalid_version_path)
        self._execute_and_assert_with_correct_result(template)

    def test_validate_template_with_invalid_version(self):
        invalid_version_path = \
            VERSION_TEMPLATE_DIR % (utils.get_resources_dir(),
                                    "invalid_version.yaml")
        template = file_utils.load_yaml_file(invalid_version_path)
        self._execute_and_assert_with_fault_result(template, 63)

    def _execute_condition_template_with_correct_result(self, template_name):
        template_path = CONDITION_TEMPLATES_DIR % (utils.get_resources_dir(),
                                                   template_name)
        template_definition = file_utils.load_yaml_file(template_path, True)
        self._execute_and_assert_with_correct_result(template_definition)

    def _execute_condition_template_with_fault_result(
            self, template_name, status_code):
        template_path = CONDITION_TEMPLATES_DIR % (utils.get_resources_dir(),
                                                   template_name)
        template_definition = file_utils.load_yaml_file(template_path, True)
        self._execute_and_assert_with_fault_result(
            template_definition, status_code)

    def _execute_and_assert_with_fault_result(self,
                                              template,
                                              status_code,
                                              def_temps=None):

        if def_temps is None:
            def_temps = {}
        result = validator.content_validation(template, def_temps)
        self._assert_fault_result(result, status_code)

    def _execute_and_assert_with_correct_result(self,
                                                template,
                                                def_temps=None):

        if def_temps is None:
            def_temps = {}

        result = validator.content_validation(template, def_temps)
        self._assert_correct_result(result)

    def _create_scenario_actions(self, target, source):

        actions = []
        raise_alarm_action = self._create_raise_alarm_action(target)
        actions.append({TemplateFields.ACTION: raise_alarm_action})

        set_state_action = self._create_set_state_action(target)
        actions.append({TemplateFields.ACTION: set_state_action})

        mark_host_down_action = self._create_mark_down_action(target)
        actions.append({TemplateFields.ACTION: mark_host_down_action})

        causal_action = self._create_add_causal_relationship_action(target,
                                                                    source)
        actions.append({TemplateFields.ACTION: causal_action})

        return actions



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import defaultdict
from itertools import groupby
from operator import itemgetter
from oslo_log import log

LOG = log.getLogger(__name__)


class CorrelationPriorities(object):
    HIGH = "HIGH"
    MEDIUM = "MEDIUM"
    LOW = "LOW"


class AlarmsProperties(object):
    ALARM1_RESOURCE_ID = 0
    ALARM1_RESOURCE_TYPE = 1
    ALARM1_NAME = 2
    ALARM2_RESOURCE_ID = 3
    ALARM2_RESOURCE_TYPE = 4
    ALARM2_NAME = 5
    OFFSET_DELTA = 6
    CORR_SCORE = 7


class CorrelationCollection(object):

    def __init__(self, high_corr_score, med_corr_score):
        self.correlation_list = []
        self.high_corr_score = high_corr_score
        self.med_corr_score = med_corr_score

    def set(self, alarm_1, alarm_2, offset_delta, correlation_score):

        self.correlation_list.append(alarm_1 + alarm_2 +
                                     (offset_delta, correlation_score))

    def get_aggregated(self):

        pairs_dict = defaultdict(list)

        for alarm_pair in self.correlation_list:
            if ((alarm_pair[AlarmsProperties.ALARM1_RESOURCE_TYPE],
                 alarm_pair[AlarmsProperties.ALARM1_NAME],
                 alarm_pair[AlarmsProperties.ALARM2_RESOURCE_TYPE],
                 alarm_pair[AlarmsProperties.ALARM2_NAME])) in pairs_dict:

                pairs_dict[alarm_pair[AlarmsProperties.ALARM1_RESOURCE_TYPE],
                           alarm_pair[AlarmsProperties.ALARM1_NAME],
                           alarm_pair[AlarmsProperties.ALARM2_RESOURCE_TYPE],
                           alarm_pair[AlarmsProperties.ALARM2_NAME]]\
                    .append(alarm_pair[AlarmsProperties.CORR_SCORE])
            else:
                pairs_dict[alarm_pair[AlarmsProperties.ALARM2_RESOURCE_TYPE],
                           alarm_pair[AlarmsProperties.ALARM2_NAME],
                           alarm_pair[AlarmsProperties.ALARM1_RESOURCE_TYPE],
                           alarm_pair[AlarmsProperties.ALARM1_NAME]].\
                    append(alarm_pair[AlarmsProperties.CORR_SCORE])

        results = [(key, sum((pairs_dict[key])) / float(len(pairs_dict[key])))
                   for key in pairs_dict]

        categorize = lambda x: CorrelationPriorities.HIGH \
            if x[1] >= self.high_corr_score \
            else CorrelationPriorities.MEDIUM \
            if x[1] >= self.med_corr_score else CorrelationPriorities.LOW

        return [(key, [(x, y) for x, y in group]) for
                key, group in groupby(sorted(results, key=itemgetter(1)),
                                      key=categorize)]



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log
import oslo_messaging

from vitrage.common.constants import NotifierEventTypes
from vitrage.messaging import get_transport


CONF = cfg.CONF
LOG = log.getLogger(__name__)


class EvaluatorNotifier(object):
    """Allows writing to message bus"""
    def __init__(self, ):
        self.oslo_notifiers = {}
        try:
            notifier_plugins = CONF.notifiers

            LOG.debug('notifier_plugins: %s', notifier_plugins)

            if not notifier_plugins:
                LOG.info('Evaluator Notifier is disabled')
                return

            topic_prefix = \
                CONF.evaluator_actions.evaluator_notification_topic_prefix

            for notifier in notifier_plugins:
                LOG.debug('Adding evaluator notifier %s', notifier)

                self.oslo_notifiers[notifier] = oslo_messaging.Notifier(
                    get_transport(),
                    driver='messagingv2',
                    publisher_id='vitrage.evaluator',
                    topics=[topic_prefix + '.' + notifier])

        except Exception:
            LOG.exception('Evaluator Notifier - missing configuration')

    @property
    def enabled(self):
        return len(self.oslo_notifiers) > 0

    def notify(self, execution_engine, properties):
        """Send a message to the wanted notifier

        :param execution_engine: the external engine that should handle the
               notification and execute an action
        :param properties: Properties to be processed by the external engine
        """

        LOG.debug('execution_engine: %s, properties: %s',
                  execution_engine,
                  properties)

        try:
            if execution_engine in self.oslo_notifiers:
                LOG.debug('Notifying %s', execution_engine)
                self.oslo_notifiers[execution_engine].info(
                    {},
                    NotifierEventTypes.EXECUTE_EXTERNAL_ACTION,
                    properties)
        except Exception:
            LOG.exception('Cannot notify - %s.',
                          NotifierEventTypes.EXECUTE_EXTERNAL_ACTION)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from oslo_config import cfg

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.collectd import COLLECTD_DATASOURCE
from vitrage.datasources.collectd.properties import \
    CollectdProperties as CProps
from vitrage.datasources.collectd.transformer import CollectdTransformer
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.host.transformer import HostTransformer
from vitrage.datasources.transformer_base import TransformerBase
from vitrage.tests.mocks import mock_transformer
from vitrage.tests.unit.datasources.test_alarm_transformer_base import \
    BaseAlarmTransformerTest
from vitrage.utils.datetime import format_unix_timestamp


# noinspection PyProtectedMember
class TestCollectdTransformer(BaseAlarmTransformerTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PUSH),
    ]

    def setUp(self):
        super(TestCollectdTransformer, self).setUp()
        self.transformers = {}
        self.transformers[COLLECTD_DATASOURCE] = \
            CollectdTransformer(self.transformers)
        self.transformers[NOVA_HOST_DATASOURCE] = \
            HostTransformer(self.transformers)

    def test_create_update_entity_vertex(self):
        # Test setup
        time1 = time.time()
        host1 = 'compute-1'
        event = self._generate_event(time1, host1, 'WARNING')
        self.assertIsNotNone(event)

        # Test action
        transformer = self.transformers[COLLECTD_DATASOURCE]
        wrapper = transformer.transform(event)

        # Test assertions
        self._validate_vertex_props(wrapper.vertex, event)

        entity_key = transformer._create_entity_key(event)
        entity_uuid = TransformerBase.uuid_from_deprecated_vitrage_id(
            entity_key)

        # Validate the neighbors: only one valid host neighbor
        self._validate_host_neighbor(wrapper, entity_uuid, host1)

        # Validate the expected action on the graph - update or delete
        self._validate_graph_action(wrapper)

        # Create an event with status 'UP'
        time2 = time.time()
        host2 = 'compute-2'
        event = self._generate_event(time2, host2, 'OK')
        self.assertIsNotNone(event)

        # Test action
        entity_key = transformer._create_entity_key(event)
        entity_uuid = \
            TransformerBase.uuid_from_deprecated_vitrage_id(entity_key)

        transformer = self.transformers[COLLECTD_DATASOURCE]
        wrapper = transformer.transform(event)

        # Test assertions
        self._validate_vertex_props(wrapper.vertex, event)
        self._validate_host_neighbor(wrapper, entity_uuid, host2)
        self._validate_graph_action(wrapper)

    def _validate_vertex_props(self, vertex, event):
        timestamp = format_unix_timestamp(event[CProps.TIME])
        self._validate_alarm_vertex_props(vertex,
                                          event[CProps.MESSAGE],
                                          COLLECTD_DATASOURCE,
                                          timestamp)

    @staticmethod
    def _generate_event(time, hostname, severity):
        # fake query result to be used by the transformer for determining
        # the neighbor
        query_result = [{VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE,
                         VProps.ID: hostname}]

        update_vals = {CProps.HOST: hostname,
                       CProps.SEVERITY: severity,
                       CProps.TIME: time,
                       DSProps.SAMPLE_DATE: format_unix_timestamp(time),
                       CProps.RESOURCE_NAME: hostname,
                       TransformerBase.QUERY_RESULT: query_result}

        generators = mock_transformer.simple_collectd_alarm_generators(
            update_vals=update_vals)

        return mock_transformer.generate_random_events_list(generators)[0]

    def _is_erroneous(self, vertex):
        return vertex[CProps.SEVERITY] != 'OK'



# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.evaluator.template_functions import function_resolver
from vitrage.evaluator.template_functions import GET_PARAM
from vitrage.evaluator.template_functions.v2.functions import get_param
from vitrage.evaluator.template_validation.base import get_custom_fault_result
from vitrage.evaluator.template_validation.base import ValidationError
from vitrage.evaluator.template_validation.content.base import \
    get_content_correct_result


class GetParamValidator(object):

    @classmethod
    def validate(cls, template, actual_params):
        try:
            function_resolver.validate_function(
                function_resolver.FuncInfo(
                    name=GET_PARAM, func=get_param, error_code=0),
                template,
                actual_params=actual_params)
        except ValidationError as e:
            return get_custom_fault_result(e.code, e.details)
        return get_content_correct_result()



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class CinderProperties(object):
    SIZE = 'size'
    VOLUME_TYPE = 'volume_type'
    ATTACHMENTS = 'attachments'
    DISPLAY_NAME = 'display_name'
    ID = 'id'
    STATUS = 'status'
    CREATED_AT = 'created_at'
    VOLUME_ATTACHMENT = 'volume_attachment'
    INSTANCE_UUID = 'instance_uuid'
    SERVER_ID = 'server_id'
    VOLUME_ID = 'volume_id'
    UPDATE_AT = 'updated_at'
    TENANT_ID = 'tenant_id'



#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from unittest import mock

from vitrage.api_handler.apis.template import TemplateApis
from vitrage.evaluator import init_template_schemas
from vitrage.tests.functional.api.v1 import FunctionalTest


VERSIONS = [
    {
        'version': 'v1',
        'status': 'SUPPORTED'
    },
    {
        'version': 'v2',
        'status': 'SUPPORTED'
    },
    {
        'version': 'v3',
        'status': 'CURRENT'
    }
]


class TemplateVersionsTest(FunctionalTest):
    def __init__(self, *args, **kwds):
        super(TemplateVersionsTest, self).__init__(*args, **kwds)
        self.auth = 'noauth'
        init_template_schemas()

    def test_get_versions(self):
        with mock.patch('pecan.request') as request:
            versions = TemplateApis().template_versions(mock.Mock())
            request.client.call.return_value = versions
            resp = self.get_json('/template/versions/')
            self.assert_list_equal(VERSIONS, resp)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import GraphAction
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.consistency import CONSISTENCY_DATASOURCE
from vitrage.datasources import OPENSTACK_CLUSTER
from vitrage.datasources import utils
from vitrage.evaluator.actions.evaluator_event_transformer \
    import VITRAGE_DATASOURCE
from vitrage.utils import datetime

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class ConsistencyEnforcer(object):

    def __init__(self,
                 entity_graph,
                 actions_callback=None):
        self.process_events = actions_callback
        self.graph = entity_graph
        self._init_datasources_to_mark_deleted()

    # noinspection PyBroadException
    def periodic_process(self):
        try:
            LOG.info('Periodic consistency check..')

            old_deleted_entities = self._find_old_deleted_entities()
            if old_deleted_entities:
                LOG.info('Consistency will remove %s deleted entities',
                         len(old_deleted_entities))
                LOG.debug('Consistency entities to remove: %s',
                          old_deleted_entities)
            events = self._to_events(old_deleted_entities,
                                     GraphAction.REMOVE_DELETED_ENTITY)
            self.process_events(events)

            stale_entities = self._find_outdated_entities_to_mark_as_deleted()
            if stale_entities:
                LOG.info('Consistency will mark %s entities as deleted',
                         len(stale_entities))
                LOG.debug('Consistency entities to mark deleted: %s',
                          stale_entities)
            events = self._to_events(stale_entities,
                                     GraphAction.DELETE_ENTITY)
            self.process_events(events)
            LOG.info('Periodic consistency check done.')
        except Exception:
            LOG.exception('Error in deleting vertices from entity_graph.')

    def _find_outdated_entities_to_mark_as_deleted(self):
        vitrage_sample_tstmp = str(datetime.datetime_delta(
            -2 * CONF.datasources.snapshots_interval))
        query = {
            'and': [
                {'!=': {VProps.VITRAGE_TYPE: VITRAGE_DATASOURCE}},
                {'<': {VProps.VITRAGE_SAMPLE_TIMESTAMP: vitrage_sample_tstmp}},
                {'==': {VProps.VITRAGE_IS_DELETED: False}},
            ]
        }

        vertices = self.graph.get_vertices(query_dict=query)
        return set(self._filter_vertices_to_be_marked_as_deleted(vertices))

    def _find_old_deleted_entities(self):
        vitrage_sample_tstmp = str(datetime.datetime_delta(
            -1 * CONF.consistency.min_time_to_delete))
        query = {
            'and': [
                {'==': {VProps.VITRAGE_IS_DELETED: True}},
                {'<': {VProps.VITRAGE_SAMPLE_TIMESTAMP: vitrage_sample_tstmp}}
            ]
        }

        vertices = self.graph.get_vertices(query_dict=query)

        return self._filter_vertices_to_be_deleted(vertices)

    def _to_events(self, vertices, action):
        for vertex in vertices:
            event = {
                DSProps.ENTITY_TYPE: CONSISTENCY_DATASOURCE,
                DSProps.DATASOURCE_ACTION: DatasourceAction.UPDATE,
                DSProps.SAMPLE_DATE: datetime.format_utcnow(),
                DSProps.EVENT_TYPE: action,
                VProps.VITRAGE_ID: vertex[VProps.VITRAGE_ID],
                VProps.ID: vertex.get(VProps.ID, None),
                VProps.VITRAGE_TYPE: vertex[VProps.VITRAGE_TYPE],
                VProps.VITRAGE_CATEGORY: vertex[VProps.VITRAGE_CATEGORY],
                VProps.IS_REAL_VITRAGE_ID: True
            }
            yield event

    @staticmethod
    def _filter_vertices_to_be_deleted(vertices):
        return list(filter(
            lambda ver:
            not (ver[VProps.VITRAGE_CATEGORY] == EntityCategory.RESOURCE and
                 ver[VProps.VITRAGE_TYPE] == OPENSTACK_CLUSTER), vertices))

    def _filter_vertices_to_be_marked_as_deleted(self, vertices):
        return list(filter(self._should_delete_vertex, vertices))

    def _should_delete_vertex(self, vertex):
        """Decide which vertices should be deleted by the consistency

        - delete all placeholder vertices, except from the openstack.cluster
        - delete vertices that their datasource is in the list
        """
        return (vertex.get(VProps.VITRAGE_IS_PLACEHOLDER) and
                not vertex[VProps.VITRAGE_TYPE] == OPENSTACK_CLUSTER) or \
               (vertex.get(VProps.VITRAGE_DATASOURCE_NAME) in
                self.datasources_to_mark_deleted)

    def _init_datasources_to_mark_deleted(self):
        self.datasources_to_mark_deleted = []

        for driver_name in CONF.datasources.types:
            driver_class = utils.get_driver_class(driver_name)
            if driver_class.should_delete_outdated_entities():
                self.datasources_to_mark_deleted.append(driver_name)

        if self.datasources_to_mark_deleted:
            LOG.info('Vertices of the following datasources will be deleted if'
                     'they become outdated: %s',
                     self.datasources_to_mark_deleted)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from collections import defaultdict
from collections import namedtuple

import itertools
from oslo_log import log

from vitrage.common.constants import TemplateStatus
from vitrage.common.constants import TemplateTypes as TType
from vitrage.common.utils import get_portion
from vitrage.evaluator.base import get_template_schema
from vitrage.evaluator.base import Template
from vitrage.evaluator.base import TEMPLATE_LOADER
from vitrage.evaluator.equivalence_repository import EquivalenceRepository
from vitrage.evaluator.template_loading.scenario_loader import ScenarioLoader
from vitrage.graph.filter import check_filter as check_subset
from vitrage import storage

LOG = log.getLogger(__name__)

EdgeKeyScenario = namedtuple('EdgeKeyScenario', ['label', 'source', 'target'])
DEF_TEMPLATES_DIR_OPT = 'def_templates_dir'


class ScenarioRepository(object):
    def __init__(self, worker_index=None, workers_num=None):
        """Create an instance of ScenarioRepository

        :param conf:
        :param worker_index: Index of the current evaluator worker
        :param workers_num: Total number of evaluator workers
        """
        self._templates = {}
        self._def_templates = {}
        self._all_scenarios = []
        self._db = storage.get_connection_from_config()
        self.entity_equivalences = EquivalenceRepository().load(self._db)
        self.relationship_scenarios = defaultdict(list)
        self.entity_scenarios = defaultdict(list)
        self._load_def_templates_from_db()
        self._load_templates_from_db()
        self._enable_worker_scenarios(worker_index, workers_num)
        self.actions = self._create_actions_collection()

    @property
    def templates(self):
        return self._templates

    @property
    def def_templates(self):
        return self._def_templates

    @def_templates.setter
    def def_templates(self, def_templates):
        self._def_templates = def_templates

    @templates.setter
    def templates(self, templates):
        self._templates = templates

    def get_scenarios_by_vertex(self, vertex):

        entity_key = vertex.properties

        scenarios = []
        for scenario_key, value in self.entity_scenarios.items():
            if check_subset(entity_key, dict(scenario_key)):
                scenarios += [(e, s) for e, s in value if s.enabled]
        return scenarios

    def get_scenarios_by_edge(self, edge_description):

        key = self._create_edge_scenario_key(edge_description)
        scenarios = []

        for scenario_key, value in self.relationship_scenarios.items():

            check_label = key.label == scenario_key.label
            if check_label \
                    and check_subset(dict(key.source),
                                     dict(scenario_key.source)) \
                    and check_subset(dict(key.target),
                                     dict(scenario_key.target)):
                scenarios += [(e, s) for e, s in value if s.enabled]

        return scenarios

    def _add_template(self, template):
        self.templates[template.uuid] = Template(template.uuid,
                                                 template.file_content,
                                                 template.created_at)
        schema = get_template_schema(template.file_content)
        template_data = schema.loaders[TEMPLATE_LOADER].load(
            schema,
            template.file_content,
            self._def_templates)
        for scenario in template_data.scenarios:
            for equivalent_scenario in self._expand_equivalence(scenario):
                self._add_scenario(equivalent_scenario)

    def _add_def_template(self, def_template):
        self.def_templates[def_template.uuid] = Template(
            def_template.uuid,
            def_template.file_content,
            def_template.created_at)

    def _expand_equivalence(self, scenario):
        equivalent_scenarios = [scenario]
        for symbol_name, entity in scenario.entities.items():
            entity_key = frozenset(entity.properties.items())
            if entity_key not in self.entity_equivalences:
                continue
            equivalent_scenarios = self._expand_on_symbol(
                equivalent_scenarios,
                symbol_name,
                self.entity_equivalences[entity_key] - {entity_key})
        return equivalent_scenarios

    @staticmethod
    def _expand_on_symbol(scenarios_in, symbol_name, entity_keys):
        scenarios_out = list(scenarios_in)
        for entity_key in entity_keys:
            for scenario in scenarios_in:
                equivalent_scenario = \
                    ScenarioLoader.build_equivalent_scenario(scenario,
                                                             symbol_name,
                                                             entity_key)
                scenarios_out.append(equivalent_scenario)
        return scenarios_out

    def _add_scenario(self, scenario):
        for entity in scenario.entities.values():
            self._add_entity_scenario(scenario, entity)
        for relationship in scenario.relationships.values():
            self._add_relationship_scenario(scenario, relationship)
        self._all_scenarios.append(scenario)

    def _load_def_templates_from_db(self):
        def_templates = self._db.templates.query(
            template_type=TType.DEFINITION,
            status=TemplateStatus.ACTIVE)
        for def_template in def_templates:
            self._add_def_template(def_template)

    def _load_templates_from_db(self):
        items = self._db.templates.query(template_type=TType.STANDARD)
        # TODO(ikinory): statuses may cause loading templates to be running
        templates = [x for x in items if x.status in [TemplateStatus.ACTIVE,
                                                      TemplateStatus.LOADING,
                                                      TemplateStatus.DELETING]]
        for t in templates:
            self._add_template(t)

    @staticmethod
    def _create_scenario_key(properties):
        return frozenset(properties)

    def _add_relationship_scenario(self, scenario, edge_desc):

        key = self._create_edge_scenario_key(edge_desc)
        self.relationship_scenarios[key].append((edge_desc, scenario))

    @staticmethod
    def _create_edge_scenario_key(edge_desc):
        try:
            source_set = frozenset(edge_desc.source.properties.items())
            target_set = frozenset(edge_desc.target.properties.items())
        except Exception as e:
            LOG.error('frozenset for edge failed - Source:%s Target:%s',
                      edge_desc.source,
                      edge_desc.target)
            raise e
        return EdgeKeyScenario(edge_desc.edge.label, source_set, target_set)

    def _add_entity_scenario(self, scenario, entity):

        key = frozenset(list(entity.properties.items()))
        self.entity_scenarios[key].append((entity, scenario))

    def _enable_worker_scenarios(self, worker_ind, n):
        """Enable a portion of the scenarios"""
        self._all_scenarios.sort(key=lambda scenario: scenario.id)

        scenarios = self._all_scenarios if \
            worker_ind is None or n is None else\
            get_portion(self._all_scenarios, n, worker_ind)
        for s in scenarios:
            s.enabled = True

    def _create_actions_collection(self):
        action_lists = (s.actions for s in self._all_scenarios)
        actions = (a for a in itertools.chain(*action_lists))
        return {a.id: a for a in actions}

    def log_enabled_scenarios(self):
        scenarios = [s for s in self._all_scenarios if s.enabled]
        if scenarios:
            LOG.info("Scenarios:\n%s", sorted([s.id for s in scenarios]))



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
import socket

from collections import namedtuple
from ipaddress import ip_address
from oslo_config import cfg
from oslo_log import log
import requests
from urllib import parse as urlparse

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EventProperties as EProps
from vitrage.datasources.alarm_driver_base import AlarmDriverBase
from vitrage.datasources.prometheus import PROMETHEUS_DATASOURCE
from vitrage.datasources.prometheus.properties import get_alarm_update_time
from vitrage.datasources.prometheus.properties import get_label
from vitrage.datasources.prometheus.properties import PrometheusAlertLabels \
    as PAlertLabels
from vitrage.datasources.prometheus.properties \
    import PrometheusAlertProperties as PAlertProps
from vitrage.datasources.prometheus.properties import PrometheusAlertStatus \
    as PAlertStatus
from vitrage.datasources.prometheus.properties \
    import PrometheusConfigFileProperties as PCFProps
from vitrage.datasources.prometheus.properties \
    import PrometheusDatasourceProperties as PDProps
from vitrage.datasources.prometheus.properties \
    import PrometheusGetAllProperties as PGAProps
from vitrage.datasources.prometheus.properties \
    import PrometheusProperties as PProps
from vitrage import os_clients
from vitrage.utils import file as file_utils

CONF = cfg.CONF
LOG = log.getLogger(__name__)

PROMETHEUS_EVENT_TYPE = 'prometheus.alarm'


class PrometheusDriver(AlarmDriverBase):
    """Handle Prometheus events.

    Prometheus driver uses a configuration file that maps
    the Prometheus alert labels to a corresponding Vitrage resource
    with specific properties (id or other unique properties).
    The mapping will most likely be defined by the alert name and other fields.

    Prometheus configuration file structure:
    The configuration file contains a list of alerts.
    Each alert contains key and resource.

    The key contains labels which uniquely identify each alert.

    The resource specifies how to identify in Vitrage the resource that
    the alert is on. It contains one or more Vitrage property names and
    corresponding Prometheus alert labels.

    Example:
    ^^^^^^^^
        Prometheus event's details:
        ---------------------------
            {
              "status": "firing",
              "version": "4",
              "groupLabels": {
                "alertname": "HighCpuOnVmAlert"
              },
              "commonAnnotations": {
                "description": "Test alert to test libvirt exporter.\n",
                "title": "High cpu usage on vm"
              },
              "groupKey": "{}:{alertname=\"HighCpuOnVmAlert\"}",
              "receiver": "vitrage",
              "externalURL": "http://vitrage.is.the.best:9093",
              "alerts": [
                {
                  "status": "firing",
                  "labels": {
                    "instance": "1.1.1.1:9999",
                    "domain": "instance-00000004",
                    "job": "libvirt",
                    "alertname": "HighCpuOnVmAlert",
                    "severity": "critical"
                  },
                  "endsAt": "2019-01-16T12:26:05.91446215Z",
                  "generatorURL": "http://seriously.vitrage.is.the.best",
                  "startsAt": "2019-01-16T12:11:50.91446215Z",
                  "annotations": {
                    "description": "Test alert to test libvirt exporter.\n",
                    "title": "High cpu usage on vm"
                  }
                },
              ],
              "commonLabels": {
                "instance": "1.1.1.1:9999",
                "job": "libvirt",
                "severity": "critical",
                "alertname": "HighCpuOnVmAlert"
              }
            }


        prometheus_conf.yaml:
        ---------------------
            alerts:
            - key:
                alertname: HighCpuOnVmAlert
                job: libvirt
              resource:
                instance_name: domain
                host_id: instance


    `enrich_event` functions are explained based on the example above.
    """

    AlarmKey = namedtuple('AlarmKey', [PAlertLabels.ALERT_NAME,
                                       PCFProps.RESOURCE])
    conf_map = {}

    def __init__(self):
        super(PrometheusDriver, self).__init__()
        self._client = None
        self._nova_client = None
        self.conf_map = self._configuration_mapping()

    @property
    def nova_client(self):
        if not self._nova_client:
            self._nova_client = os_clients.nova_client()
        return self._nova_client

    def _vitrage_type(self):
        return PROMETHEUS_DATASOURCE

    def _alarm_key(self, alert):
        return self.AlarmKey(
            alertname=get_label(alert, PAlertLabels.ALERT_NAME),
            resource=str(self._get_resource_alert_values(alert)))

    def _is_erroneous(self, alert):
        return alert and PAlertStatus.FIRING == alert.get(PAlertProps.STATUS)

    def _is_valid(self, alert):
        if not alert or PAlertProps.STATUS not in alert:
            return False
        return True

    def _status_changed(self, new_alarm, old_alarm):
        return \
            new_alarm.get(PAlertProps.STATUS) != \
            old_alarm.get(PAlertProps.STATUS)

    def _get_all_alarms(self):
        alertmanager_url = CONF.prometheus.alertmanager_url
        receiver = CONF.prometheus.receiver
        if not alertmanager_url:
            LOG.warning('Alertmanager url is not defined')
            return []

        if not receiver:
            LOG.warning('Receiver is not defined')
            return []

        payload = {PGAProps.ACTIVE: 'true',
                   PGAProps.RECEIVER: receiver}

        session = requests.Session()

        response = session.get(alertmanager_url,
                               params=payload)

        if response.status_code == requests.codes.ok:
            if 'v1' in alertmanager_url:
                alerts = json.loads(response.text)[PGAProps.DATA]
            else:
                alerts = json.loads(response.text)
            self._modify_alert_status(alerts)
            alarms = self._enrich_alerts(alerts, PROMETHEUS_EVENT_TYPE)
            return alarms
        else:
            LOG.error('Failed to get Alertmanager data. Response code: %s',
                      response.status_code)
        return []

    @staticmethod
    def _modify_alert_status(alerts):
        for alert in alerts:
            if alert.get(PAlertProps.STATUS).get(PGAProps.STATE) == \
                    PGAProps.ACTIVE:
                alert[PAlertProps.STATUS] = PAlertStatus.FIRING

    def _get_changed_alarms(self):
        return []

    @staticmethod
    def _configuration_mapping():
        prometheus_config_file = CONF.prometheus[DSOpts.CONFIG_FILE]
        try:
            prometheus_config = \
                file_utils.load_yaml_file(prometheus_config_file)
            return prometheus_config[PCFProps.ALERTS]
        except Exception:
            LOG.exception('Failed in init the configuration file: %s',
                          prometheus_config_file)
            return {}

    def enrich_event(self, event, event_type):
        """Get an alert event from Prometheus and create a list of alert events

        :param event: Prometheus event.
        :param event_type: The type of the event. Always 'prometheus.alert'.
        :return: a list of alarms, one per Prometheus alert

        For the example above. The function returns:
            {
              "status": "firing",
              "labels": {
                "instance": "1.1.1.1:9999",
                "domain": "instance-00000004",
                "job": "libvirt",
                "alertname": "HighCpuOnVmAlert",
                "severity": "critical"
              },
              "vitrage_entity_type": "prometheus",
              "endsAt": "2019-01-16T12:39:50.91446215Z",
              "generatorURL": "http://seriously.vitrage.is.the.best",
              "vitrage_datasource_name": "prometheus",
              "startsAt": "2019-01-16T12:11:50.91446215Z",
              "vitrage_datasource_action": "update",
              "vitrage_entity_unique_props": {
                "instance_name": "instance-00000004",
                "host_id": "my-host-name"
              },
              "vitrage_sample_date": "2019-01-16T13:10:33Z",
              "vitrage_event_type": "prometheus.alarm",
              "annotations": {
                "description": "Test alert to test libvirt exporter.\n",
                "title": "High cpu usage on vm"
              }
            }

        """

        LOG.debug('Going to enrich event: %s', event)

        alarms = []
        details = event.get(EProps.DETAILS)
        if details:
            alarms = self._enrich_alerts(details.get(PProps.ALERTS, []),
                                         event_type)

        LOG.debug('Enriched event. Created alert events: %s', alarms)

        return self.make_pickleable(alarms, PROMETHEUS_DATASOURCE,
                                    DatasourceAction.UPDATE)

    def _enrich_alerts(self, alerts, event_type):
        return [self._enrich_alert(alert, event_type) for alert in alerts]

    def _enrich_alert(self, alert, event_type):
        """Enrich prometheus alert.

        Adding fields to prometheus alert in order to map it to vitrage entity.

        :param alert: Prometheus alert
        :param event_type: The type of the event. Always 'prometheus.alert'.
        :return: Enriched prometheus alert
        """
        alert[DSProps.EVENT_TYPE] = event_type
        vitrage_entity_unique_props = \
            self._calculate_vitrage_entity_unique_props(alert)
        alert[PDProps.ENTITY_UNIQUE_PROPS] = \
            vitrage_entity_unique_props
        old_alarm = self._old_alarm(alert)
        alert = self._filter_and_cache_alarm(
            alert, old_alarm,
            self._filter_get_erroneous,
            get_alarm_update_time(alert))
        return alert

    def _calculate_vitrage_entity_unique_props(self, alert):
        """Build a vitrage entity unique props.

        The unique props are based on the alert and the conf file.

        :param alert: Prometheus alert
        :type alert: dict
        :return: Unique properties of vitrage entity
        ":rtype: dict

        For the example above. The function returns:
            {'instance_name': 'instance-00000004',
            'host_id': 'my-host-name'}
        """
        resource_labels = self._get_conf_resource(alert)
        vitrage_entity_unique_props = {}
        for vitrage_label in resource_labels:
            prometheus_label = resource_labels[vitrage_label]
            label_value = str(get_label(alert, prometheus_label))
            vitrage_entity_unique_props[vitrage_label] = \
                self._adjust_label_value(label_value)
        return vitrage_entity_unique_props

    def _adjust_label_value(self, label_value):
        """Adjust the given value of the alert's label

        First check if the value is ip.
        Then, get its hostname if it has one.
        If not, fetch the instance id from nova by its ip.
        Otherwise, leave the label value as is.

        :param label_value: Value of alert's label
        :type label_value: str
        :return: Adjusted label's value of the alert as described.
        :rtype: str

        For the example above. The function returns:
          - label_value='instance-00000004' it returns:'instance-00000004'
          - label_value='1.1.1.1:9999' it returns:'my-host-name'
        """
        if label_value is not None:
            try:
                # Check if the value is ip
                ip = str(self._validate_ip(label_value))
                try:
                    # Get hostname of the ip
                    entity_hostname = socket.gethostbyaddr(ip)
                    label_value = entity_hostname[0]

                except socket.error:
                    # If not ip of a host
                    nova_instance = self.nova_client.servers.list(
                        search_opts={'all_tenants': 1, 'ip': ip})
                    if nova_instance:
                        label_value = nova_instance[0].id
                    else:
                        label_value = ip

            except ValueError:
                # If not ip value, leave it as is
                pass

        return label_value

    def _get_resource_alert_values(self, alert):
        """Get values of the alert labels from alert's resource in config file.

        For the example above. The function returns:
            {'instance': '1.1.1.1:9999', 'domain': 'instance-00000004'}
        """

        resource_alert_labels = self._get_conf_resource(alert).values()
        alert_values = {label: get_label(alert, label)
                        for label in resource_alert_labels}
        return alert_values

    def _get_conf_resource(self, alert):
        """Get resource from conf file that matches the alert.

        Matching a resource from conf file to alert is done by
        alert's key in the conf file.
        The alert's key in conf file contains alert's labels and
        their value as in Prometheus alert.

        :param alert: Prometheus alert
        :type alert: dict
        :return: Resource that matches the alert
        :rtype: dict

        Resource is a dict, where the keys are vitrage entity fields
        and its values are the corresponding alert labels.

        For the example above. The function returns:
          {'instance_name': 'domain', 'host_id': 'instance'}
        """
        if self.conf_map:
            for conf_alert in self.conf_map:
                alert_key = conf_alert[PCFProps.KEY].items()
                alert_labels = alert[PAlertProps.LABELS].items()
                match = set(alert_key).issubset(set(alert_labels))
                if match:
                    return conf_alert[PCFProps.RESOURCE]
        return {}

    @staticmethod
    def _validate_ip(value):
        """Check if the value is ip address.

        If the value is in ip:port form, separate it and validate just the ip.

        :param value: String value
        :return:An IPv4Address or IPv6Address object
        :raises ValueError: if the *value* passed isn't either a v4 or a v6
        address
        """
        # check if the value is ip
        try:
            ip = ip_address(str(value))
        except ValueError:
            parsed = urlparse.urlparse('//{}'.format(value))
            ip = ip_address(str(parsed.hostname))
        return ip

    @staticmethod
    def get_event_types():
        return [PROMETHEUS_EVENT_TYPE]

    @staticmethod
    def should_delete_outdated_entities():
        return True



# Copyright 2017 - ZTE
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

import cotyledon

from vitrage.cli import VITRAGE_TITLE
from vitrage.common import config
from vitrage.snmp_parsing.service import SnmpParsingService


def main():
    print(VITRAGE_TITLE)
    config.parse_config(sys.argv)
    sm = cotyledon.ServiceManager()
    sm.add(SnmpParsingService)
    sm.run()


if __name__ == "__main__":
    sys.exit(main())



# Copyright 2015 - Alcatel-Lucent
# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import importutils

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.exception import VitrageTransformerError
from vitrage.datasources.consistency import CONSISTENCY_DATASOURCE
from vitrage.datasources.consistency.transformer import ConsistencyTransformer
from vitrage.evaluator.actions.evaluator_event_transformer import \
    EvaluatorEventTransformer
from vitrage.evaluator.actions.evaluator_event_transformer \
    import VITRAGE_DATASOURCE
from vitrage.utils import opt_exists

CONF = cfg.CONF
LOG = logging.getLogger(__name__)
ENTITIES = 'entities'


class TransformerManager(object):

    def __init__(self):
        self.transformers = self.register_transformer_classes()

    @staticmethod
    def register_transformer_classes():

        transformers = {}
        for datasource_type in CONF.datasources.types:
            try:
                transformers[datasource_type] = importutils.import_object(
                    CONF[datasource_type].transformer,
                    transformers)
                if opt_exists(CONF[datasource_type], ENTITIES):
                    for entity in CONF[datasource_type].entities:
                        transformers[entity] = importutils.import_object(
                            CONF[datasource_type].transformer,
                            transformers)

            except Exception:
                LOG.exception('Failed to register transformer %s.',
                              datasource_type)

        transformers[VITRAGE_DATASOURCE] = importutils.import_object(
            "%s.%s" % (EvaluatorEventTransformer.__module__,
                       EvaluatorEventTransformer.__name__), transformers)

        transformers[CONSISTENCY_DATASOURCE] = importutils.import_object(
            "%s.%s" % (ConsistencyTransformer.__module__,
                       ConsistencyTransformer.__name__), transformers)

        return transformers

    def get_transformer(self, key):
        try:
            transformer = self.transformers[key]
        except KeyError:
            raise VitrageTransformerError(
                'Could not get transformer instance for %s' % key)

        return transformer

    def transform(self, entity_event):
        entity_type = self.get_entity_type(entity_event)
        return self.get_transformer(entity_type).transform(entity_event)

    def get_enrich_query(self, entity_event):
        entity_type = self.get_entity_type(entity_event)
        return self.get_transformer(entity_type).get_enrich_query(entity_event)

    @staticmethod
    def get_entity_type(entity_event):
        try:
            return entity_event[DSProps.ENTITY_TYPE]
        except KeyError:
            raise VitrageTransformerError(
                'Entity Event must contains entity_type field.')



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import GraphAction
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.consistency import CONSISTENCY_DATASOURCE
from vitrage.datasources.consistency.transformer import ConsistencyTransformer
from vitrage.tests import base

from vitrage.tests.mocks import mock_driver as mock_sync

LOG = logging.getLogger(__name__)


class TestConsistencyTransformer(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PUSH),
    ]

    # noinspection PyAttributeOutsideInit,PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(TestConsistencyTransformer, cls).setUpClass()
        cls.transformers = {}
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=CONSISTENCY_DATASOURCE)
        cls.transformers[CONSISTENCY_DATASOURCE] = \
            ConsistencyTransformer(cls.transformers)
        cls.actions = [GraphAction.DELETE_ENTITY,
                       GraphAction.REMOVE_DELETED_ENTITY]

    def test_snapshot_transform(self):
        LOG.debug('Consistency transformer test: transform entity event '
                  'snapshot')

        # Test setup
        spec_list = mock_sync.simple_consistency_generators(consistency_num=7,
                                                            update_events=7)
        static_events = mock_sync.generate_random_events_list(spec_list)

        for event in static_events:
            # Test action
            wrapper = self.transformers[CONSISTENCY_DATASOURCE].transform(
                event)

            # Test assertions
            vertex = wrapper.vertex
            self._validate_consistency_vertex_props(vertex, event)

            neighbors = wrapper.neighbors
            self.assertIsNone(neighbors)

            action = wrapper.action
            self.assertIn(action, self.actions)

    def test_update_transform(self):
        LOG.debug('Consistency transformer test: transform entity event '
                  'update')

        # Test setup
        spec_list = mock_sync.simple_consistency_generators(consistency_num=7,
                                                            update_events=7)
        static_events = mock_sync.generate_random_events_list(spec_list)

        for event in static_events:
            # Test action
            wrapper = self.transformers[CONSISTENCY_DATASOURCE].transform(
                event)

            # Test assertions
            vertex = wrapper.vertex
            self._validate_consistency_vertex_props(vertex, event)

            neighbors = wrapper.neighbors
            self.assertIsNone(neighbors)

            action = wrapper.action
            self.assertIn(action, self.actions)

    def _validate_consistency_vertex_props(self, vertex, event):
        vitrage_id = event.get(VProps.VITRAGE_ID, None)
        self.assertIsNotNone(vitrage_id)

        vertex_id = vertex.vertex_id
        self.assertIsNotNone(vertex_id)

        vitrage_sample_timestamp = \
            vertex.get(VProps.VITRAGE_SAMPLE_TIMESTAMP, None)
        self.assertIsNotNone(vitrage_sample_timestamp)



# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from vitrage.tests.functional.api_handler.test_templates import TestTemplates

TEMPLATE_WITH_PARAMS = 'v2_with_params.yaml'
TEMPLATE_WITH_EXTRA_PARAM_DEF = 'v2_with_extra_param_def.yaml'
TEMPLATE_WITH_MISSING_PARAM_DEF = 'v2_with_missing_param_def.yaml'
TEMPLATE_WITHOUT_PARAMS = 'v2_without_params.yaml'


class TestTemplatesV2(TestTemplates):

    def test_validate_template_with_no_params(self):
        self._validate_template_with_no_params(TEMPLATE_WITH_PARAMS)

    def test_validate_template_with_missing_param(self):
        self._validate_template_with_missing_param(TEMPLATE_WITH_PARAMS)

    def test_validate_template_with_actual_params(self):
        self._validate_template_with_actual_params(TEMPLATE_WITH_PARAMS)

    def test_validate_template_with_missing_param_def(self):
        self._validate_template_with_missing_param_def(
            TEMPLATE_WITH_MISSING_PARAM_DEF)

    def test_validate_template_without_params(self):
        self._validate_template_without_params(TEMPLATE_WITHOUT_PARAMS)

    def test_validate_template_with_extra_actual_param(self):
        self._validate_template_with_extra_actual_param(TEMPLATE_WITH_PARAMS)

    def test_validate_template_with_extra_param_def(self):
        self._validate_template_with_extra_param_def(
            TEMPLATE_WITH_EXTRA_PARAM_DEF)

    def test_add_template_with_no_params(self):
        self._add_template_with_no_params(TEMPLATE_WITH_PARAMS)

    def test_add_template_with_missing_param(self):
        self._add_template_with_missing_param(TEMPLATE_WITH_PARAMS)

    def test_add_template_with_actual_params(self):
        self._add_template_with_actual_params(TEMPLATE_WITH_PARAMS)

    def test_add_template_with_missing_param_def(self):
        self._add_template_with_missing_param_def(
            TEMPLATE_WITH_MISSING_PARAM_DEF)

    def test_add_template_without_params(self):
        self._add_template_without_params(TEMPLATE_WITHOUT_PARAMS)

    def test_add_template_with_extra_actual_param(self):
        self._add_template_with_extra_actual_param(TEMPLATE_WITH_PARAMS)

    def test_add_template_with_extra_param_def(self):
        self._add_template_with_extra_param_def(TEMPLATE_WITH_EXTRA_PARAM_DEF)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log
import oslo_messaging
from oslo_utils import importutils

from vitrage.coordination import service as coord
from vitrage import messaging
from vitrage.opts import register_opts

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class VitrageNotifierService(coord.Service):

    def __init__(self, worker_id):
        super(VitrageNotifierService, self).__init__(worker_id)
        self.notifiers = self.get_notifier_plugins()
        self._init_listeners()

    def run(self):
        super(VitrageNotifierService, self).run()

        LOG.info("Vitrage Notifier Service - Starting...")

        for listener in self.listeners:
            listener.start()

        LOG.info("Vitrage Notifier Service - Started!")

    def terminate(self):
        super(VitrageNotifierService, self).terminate()
        LOG.info("Vitrage Notifier Service - Stopping...")

        for listener in self.listeners:
            listener.stop()
            listener.wait()

        LOG.info("Vitrage Notifier Service - Stopped!")

    @staticmethod
    def get_notifier_plugins():
        notifiers = []
        conf_notifier_names = CONF.notifiers
        if not conf_notifier_names:
            LOG.info('There are no notifier plugins in configuration')
            return []
        for notifier_name in conf_notifier_names:
            register_opts(notifier_name, CONF.notifiers_path)
            LOG.info('Notifier plugin %s started', notifier_name)
            notifiers.append(importutils.import_object(
                CONF[notifier_name].notifier))
        return notifiers

    def _init_listeners(self):
        self.listeners = []
        transport = messaging.get_transport()

        self._init_notifier(transport=transport,
                            topic=CONF.entity_graph.notifier_topic,
                            endpoint=VitrageDefaultEventEndpoint(
                                self.notifiers))

        topic_prefix = \
            CONF.evaluator_actions.evaluator_notification_topic_prefix

        for notifier in self.notifiers:
            if notifier.use_private_topic():
                self._init_notifier(transport=transport,
                                    topic=topic_prefix + '.' +
                                    notifier.get_notifier_name(),
                                    endpoint=notifier)

    def _init_notifier(self, transport, topic, endpoint):
        LOG.debug('Initializing notifier with topic %s', topic)

        self.listeners.append(messaging.get_notification_listener(
            transport, [oslo_messaging.Target(topic=topic)], [endpoint]))


class VitrageDefaultEventEndpoint(object):

    def __init__(self, notifiers):
        self.notifiers = notifiers

    def info(self, ctxt, publisher_id, event_type, payload, metadata):
        """Endpoint for alarm notifications"""
        LOG.info('Vitrage Event Info: publisher_id %s', publisher_id)
        LOG.info('Vitrage Event Info: event_type %s', event_type)
        LOG.info('Vitrage Event Info: metadata %s', metadata)
        LOG.info('Vitrage Event Info: payload %s', payload)
        for plugin in self.notifiers:
            plugin.process_event(payload, event_type)



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg
from oslo_log import log as logging

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps


from vitrage.datasources.resource_transformer_base import \
    ResourceTransformerBase
from vitrage.datasources.transformer_base import extract_field_value
import vitrage.graph.utils as graph_utils

from vitrage.datasources import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources import transformer_base as tbase

from vitrage.datasources.kubernetes.properties import KUBERNETES_DATASOURCE
from vitrage.datasources.kubernetes.properties import \
    KubernetesProperties as kubProp
from vitrage.utils import file as file_utils

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class KubernetesTransformer(ResourceTransformerBase):

    def _create_vertex(self, entity_event):
        metadata = {
            VProps.NAME: self._get_cluster_name(),
        }

        entity_key = self._create_entity_key(entity_event)

        vitrage_sample_timestamp = entity_event[DSProps.SAMPLE_DATE]

        update_timestamp = self._format_update_timestamp(
            extract_field_value(entity_event, DSProps.SAMPLE_DATE),
            vitrage_sample_timestamp)

        return graph_utils.create_vertex(
            entity_key,
            vitrage_category=EntityCategory.RESOURCE,
            vitrage_type=KUBERNETES_DATASOURCE,
            vitrage_sample_timestamp=vitrage_sample_timestamp,
            update_timestamp=update_timestamp,
            metadata=metadata)

    def _create_snapshot_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_update_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_snapshot_neighbors(self, entity_event):
        return self._create_node_neighbors(entity_event)

    def _create_update_neighbors(self, entity_event):
        return self._create_node_neighbors(entity_event)

    def _create_entity_key(self, event):

        key_fields = self._key_values(KUBERNETES_DATASOURCE,
                                      self._get_cluster_name())
        key = tbase.build_key(key_fields)
        return key

    def get_vitrage_type(self):
        return KUBERNETES_DATASOURCE

    def _get_cluster_name(self):
        kubeconf = file_utils.load_yaml_file(CONF.kubernetes.config_file)
        if not kubeconf:
            return None
        contexts = kubeconf['contexts']
        for context in contexts:
            if context['name'] == kubeconf['current-context']:
                cluster_name = context['context']['cluster']
        return cluster_name

    def _create_node_neighbors(self, entity_event):
        """neighbors are existing Nova instances only"""
        neighbors = []
        for neighbor in entity_event[kubProp.RESOURCES]:
            neighbor[DSProps.ENTITY_TYPE] = entity_event[DSProps.ENTITY_TYPE]
            neighbor[DSProps.DATASOURCE_ACTION] = \
                entity_event[DSProps.DATASOURCE_ACTION]
            neighbor[DSProps.SAMPLE_DATE] = entity_event[DSProps.SAMPLE_DATE]

            neighbor_id = neighbor[kubProp.EXTERNALID]
            neighbor_datasource_type = NOVA_INSTANCE_DATASOURCE
            neighbors.append(self._create_neighbor(neighbor,
                                                   neighbor_id,
                                                   neighbor_datasource_type,
                                                   EdgeLabel.COMPRISED,
                                                   is_entity_source=True))

        return neighbors



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import matchers

from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.aodh import AODH_DATASOURCE
from vitrage.datasources.nagios import NAGIOS_DATASOURCE
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources.nova.zone import NOVA_ZONE_DATASOURCE
from vitrage.entity_graph.mappings.datasource_info_mapper import \
    DatasourceInfoMapper
from vitrage.entity_graph.mappings.operational_resource_state import \
    OperationalResourceState
from vitrage.graph.utils import create_vertex
from vitrage.opts import register_opts
from vitrage.tests import base
from vitrage.tests.mocks import utils


class TestDatasourceInfoMapper(base.BaseTest):

    def _load_datasources(self):
        for datasource_name in self.conf.datasources.types:
            register_opts(datasource_name, self.conf.datasources.path)

    def setUp(self):
        super(TestDatasourceInfoMapper, self).setUp()
        self.cfg_fixture.config(
            group='entity_graph',
            datasources_values_dir=utils.get_resources_dir() +
            '/datasources_values')
        self.cfg_fixture.config(group='datasources',
                                types=[
                                    NAGIOS_DATASOURCE,
                                    NOVA_HOST_DATASOURCE,
                                    NOVA_INSTANCE_DATASOURCE,
                                    NOVA_ZONE_DATASOURCE,
                                    AODH_DATASOURCE])
        self._load_datasources()

    def test_load_datasource_value_without_errors(self):
        # action
        info_mapper = DatasourceInfoMapper()

        # test assertions

        # Total datasources plus the evaluator which is not definable
        total_datasources = len(self.conf.datasources.types) + 1
        self.assertThat(info_mapper.datasources_value_confs,
                        matchers.HasLength(total_datasources))

    def test_load_datasources_value_with_errors(self):
        # setup
        self.cfg_fixture.config(
            group='entity_graph',
            datasources_values_dir=utils.get_resources_dir() +
            '/datasources_values/erroneous_values'
        )
        self._load_datasources()

        # action
        info_mapper = DatasourceInfoMapper()

        # test assertions
        missing_values = 1
        erroneous_values = 1
        num_valid_datasources = len(info_mapper.datasources_value_confs) + \
            missing_values + erroneous_values
        self.assertThat(self.conf.datasources.types,
                        matchers.HasLength(num_valid_datasources))

    def test_vitrage_operational_value_exists(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        vitrage_operational_value = \
            info_mapper.vitrage_operational_value(NOVA_INSTANCE_DATASOURCE,
                                                  'BUILDING')

        # test assertions
        self.assertEqual(OperationalResourceState.TRANSIENT,
                         vitrage_operational_value)

    def test_vitrage_operational_value_not_exists(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        vitrage_operational_value = \
            info_mapper.vitrage_operational_value(NOVA_INSTANCE_DATASOURCE,
                                                  'NON EXISTING STATE')

        # test assertions
        self.assertEqual(OperationalResourceState.NA,
                         vitrage_operational_value)

    def test_vitrage_operational_value_DS_not_exists_and_value_not_exist(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        vitrage_operational_value = \
            info_mapper.vitrage_operational_value('NON EXISTING DATASOURCE',
                                                  'BUILDING')

        # test assertions
        self.assertEqual(OperationalResourceState.NA,
                         vitrage_operational_value)

    def test_vitrage_operational_value_DS_not_exists_and_value_exist(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        vitrage_operational_value = \
            info_mapper.vitrage_operational_value('NON EXISTING DATASOURCE',
                                                  'AVAILABLE')

        # test assertions
        self.assertEqual(OperationalResourceState.OK,
                         vitrage_operational_value)

    def test_value_priority(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        value_priority = \
            info_mapper.value_priority(NOVA_INSTANCE_DATASOURCE, 'ACTIVE')

        # test assertions
        self.assertEqual(10, value_priority)

    def test_value_priority_not_exists(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        value_priority = \
            info_mapper.value_priority(NOVA_INSTANCE_DATASOURCE,
                                       'NON EXISTING STATE')

        # test assertions
        self.assertEqual(0, value_priority)

    def test_value_priority_datasource_not_exists(self):
        # setup
        info_mapper = DatasourceInfoMapper()

        # action
        value_priority = \
            info_mapper.value_priority('NON EXISTING DATASOURCE',
                                       'ACTIVE')

        # test assertions
        self.assertEqual(10,
                         value_priority)

    def test_vitrage_aggregated_value(self):
        # setup
        info_mapper = DatasourceInfoMapper()
        metadata1 = {VProps.VITRAGE_STATE: 'SUSPENDED'}
        new_vertex1 = create_vertex('12345',
                                    vitrage_category=EntityCategory.RESOURCE,
                                    vitrage_type=NOVA_INSTANCE_DATASOURCE,
                                    entity_state='ACTIVE',
                                    metadata=metadata1)
        metadata2 = {VProps.VITRAGE_STATE: 'ACTIVE'}
        new_vertex2 = create_vertex('23456',
                                    vitrage_category=EntityCategory.RESOURCE,
                                    vitrage_type=NOVA_INSTANCE_DATASOURCE,
                                    entity_state='SUSPENDED',
                                    metadata=metadata2)

        # action
        info_mapper.vitrage_aggregate_values(new_vertex1, None)
        info_mapper.vitrage_aggregate_values(new_vertex2, None)

        # test assertions
        self.assertEqual('SUSPENDED',
                         new_vertex1[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.SUBOPTIMAL,
                         new_vertex1[VProps.VITRAGE_OPERATIONAL_STATE])
        self.assertEqual('SUSPENDED',
                         new_vertex2[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.SUBOPTIMAL,
                         new_vertex2[VProps.VITRAGE_OPERATIONAL_STATE])

    def test_vitrage_aggregated_value_functionalities(self):
        # setup
        info_mapper = DatasourceInfoMapper()
        new_vertex1 = create_vertex('12345',
                                    vitrage_category=EntityCategory.RESOURCE,
                                    vitrage_type=NOVA_INSTANCE_DATASOURCE,
                                    entity_state='ACTIVE')
        metadata2 = {VProps.VITRAGE_STATE: OperationalResourceState.SUBOPTIMAL}
        new_vertex2 = create_vertex('23456',
                                    vitrage_category=EntityCategory.RESOURCE,
                                    vitrage_type=NOVA_INSTANCE_DATASOURCE,
                                    metadata=metadata2)
        metadata3 = {VProps.VITRAGE_STATE: OperationalResourceState.SUBOPTIMAL}
        new_vertex3 = create_vertex('34567',
                                    vitrage_category=EntityCategory.RESOURCE,
                                    vitrage_type=NOVA_INSTANCE_DATASOURCE,
                                    entity_state='ACTIVE')
        graph_vertex3 = create_vertex('34567',
                                      vitrage_category=EntityCategory.RESOURCE,
                                      vitrage_type=NOVA_INSTANCE_DATASOURCE,
                                      entity_state='SUSPENDED',
                                      metadata=metadata3)

        # action
        info_mapper.vitrage_aggregate_values(new_vertex1,
                                             None)
        info_mapper.vitrage_aggregate_values(new_vertex2,
                                             None)
        info_mapper.vitrage_aggregate_values(new_vertex3,
                                             graph_vertex3)

        # test assertions
        self.assertEqual('ACTIVE',
                         new_vertex1[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.OK,
                         new_vertex1[VProps.VITRAGE_OPERATIONAL_STATE])
        self.assertEqual(OperationalResourceState.SUBOPTIMAL,
                         new_vertex2[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.SUBOPTIMAL,
                         new_vertex2[VProps.VITRAGE_OPERATIONAL_STATE])
        self.assertEqual(OperationalResourceState.SUBOPTIMAL,
                         new_vertex3[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.SUBOPTIMAL,
                         new_vertex3[VProps.VITRAGE_OPERATIONAL_STATE])

    def test_vitrage_aggregated_value_datasource_not_exists(self):
        # setup
        info_mapper = DatasourceInfoMapper()
        metadata = {VProps.VITRAGE_STATE: 'SUSPENDED'}
        new_vertex = create_vertex('12345',
                                   vitrage_category=EntityCategory.RESOURCE,
                                   vitrage_type='NON EXISTING DATASOURCE',
                                   entity_state='ACTIVE',
                                   metadata=metadata)

        # action
        info_mapper.vitrage_aggregate_values(new_vertex, None)

        # test assertions
        self.assertEqual('ACTIVE',
                         new_vertex[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.OK,
                         new_vertex[VProps.VITRAGE_OPERATIONAL_STATE])

    def test_vitrage_aggregated_value_DS_not_exists_and_wrong_state(self):
        # setup
        info_mapper = DatasourceInfoMapper()
        metadata = {VProps.VITRAGE_STATE: 'SUSPENDED'}
        new_vertex = create_vertex('12345',
                                   vitrage_category=EntityCategory.RESOURCE,
                                   vitrage_type='NON EXISTING DATASOURCE',
                                   entity_state='NON EXISTING STATE',
                                   metadata=metadata)

        # action
        info_mapper.vitrage_aggregate_values(new_vertex, None)

        # test assertions
        self.assertEqual('NON EXISTING STATE',
                         new_vertex[VProps.VITRAGE_AGGREGATED_STATE])
        self.assertEqual(OperationalResourceState.NA,
                         new_vertex[VProps.VITRAGE_OPERATIONAL_STATE])
